@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated. All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  */
@@ -140,7 +140,7 @@ vlogit ( FILE        *str,       /* STREAM to write to, or NULL for syslog */
 {
     char            *date_time = NULL;
     struct timeval   tval;
-    size_t           iLeft     = 0;
+    int              iLeft     = 0;
     int              iChunk    = 0;
     int              iDateLen  = 0;
     long             lMsec     = 0;
@@ -161,8 +161,8 @@ vlogit ( FILE        *str,       /* STREAM to write to, or NULL for syslog */
         char whence [ 512 ];
         int  len;
         
-        iLeft -= ( iChunk >= 0 ? iChunk : strlen(msgbuf) );
-        len    = Qsprintf ( whence, " [%s:%lu]", fn, ln );
+        iLeft -= ( iChunk >= 0 ? iChunk : (int) strlen(msgbuf) );
+        len    = Qsprintf ( whence, " [%s:%zu]", fn, ln );
         strncat ( msgbuf, whence, iLeft );
         msgbuf [ sizeof(msgbuf) -1 ] = '\0'; /* just to make sure */
         iLeft -= len;
