@@ -3,7 +3,7 @@
  *
  * A change-password server.
  *
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file license.txt specifies the terms for use, modification,
  * and redistribution.
  *
@@ -265,6 +265,8 @@ void    err_dump ( WHENCE, const char *format, ... );
 void    my_perror   ( void );
 char   *sys_err_str ( void );
 void    get_client_info ( POP *p, BOOL no_rev_lookup );
+char   *sock_ntop ( struct sockaddr *p, int salen );
+int     sock_port ( struct sockaddr *p, int salen );
 char   *debug_str ( char *p, int inLen, int order );
 
 
@@ -1358,9 +1360,18 @@ sys_err_str()
 
 void get_client_info ( POP *p, BOOL no_rev_lookup )
 {
+#ifdef INET6
+    struct addrinfo         hints;
+    struct addrinfo     *   res;
+    struct addrinfo     *   r;
+    struct sockaddr_storage cs;                 /*  Communication parameters */
+    int                     error;
+    char                    hbuf[NI_MAXHOST];
+#else /* INET6 */
     struct hostent      *   hp  = NULL;
     struct sockaddr_in      cs;                 /*  Communication parameters */
     struct hostent      *   ch;                 /*  Client host information */
+#endif /* INET6 */
     int                     len = 0;
     int                     sp  = 0;
 
@@ -1369,6 +1380,35 @@ void get_client_info ( POP *p, BOOL no_rev_lookup )
     p->ipaddr = strdup("* _DEBUG *");
     p->client = p->ipaddr;
 #else
+#ifdef INET6
+    memset ( &hints, 0, sizeof ( hints ) );
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags = AI_PASSIVE | AI_CANONNAME;
+    res = NULL;
+    error = getaddrinfo ( p->myhost, NULL, &hints, &res );
+    if ( !error && res->ai_canonname != NULL ) {
+        if ( index ( res->ai_canonname, '.' ) == NULL ) {         /* FQN not returned */
+            /*
+             * SVR4 resolver is stupid and returns h_name as whatever
+             * you gave gethostbyname.  Thus do a reverse lookup
+             * on the first address and hope for the best.
+             */
+	    error = getnameinfo ( res->ai_addr, res->ai_addrlen,
+				  hbuf, sizeof(hbuf), NULL, 0, NI_NAMEREQD );
+            if ( !error ) {
+		free ( p->myhost );
+		p->myhost = strdup ( hbuf );
+            }
+        } /* index ( hp->h_name, '.' ) == NULL */
+        else {
+	    free ( p->myhost );
+	    p->myhost = strdup ( res->ai_canonname );
+        }
+    }
+    if ( res )
+	freeaddrinfo ( res );
+#else /* INET6 */
     hp = gethostbyname ( p->myhost );
     if ( hp != NULL ) {
         if ( index ( hp->h_name, '.' ) == NULL ) {         /* FQN not returned */
@@ -1389,6 +1429,7 @@ void get_client_info ( POP *p, BOOL no_rev_lookup )
             p->myhost [ MAXHOSTNAMELEN ] = '\0';
         }
     }
+#endif /* INET6 */
 
     /*  
      * Get the address and socket of the client to whom I am speaking 
@@ -1398,16 +1439,36 @@ void get_client_info ( POP *p, BOOL no_rev_lookup )
         err_msg ( HERE, "Unable to obtain socket and address of client" );
     }
 
+#ifdef INET6
+    if ( ((struct sockaddr *)&cs)->sa_family == AF_INET6 &&
+	 IN6_IS_ADDR_V4MAPPED( &((struct sockaddr_in6 *)&cs)->sin6_addr ) ) {
+	struct sockaddr_in sin;
+	struct sockaddr_in6 *sin6;
+	const int off = sizeof(struct in6_addr) - sizeof(struct in_addr);
+
+	sin6 = (struct sockaddr_in6 *)&cs;
+	sin.sin_family = AF_INET;
+	len = sizeof(struct sockaddr_in);
+#ifdef SIN6_LEN
+	sin.sin_len = len;
+#endif
+	memcpy ( &sin.sin_addr, &sin6->sin6_addr.s6_addr[off],
+		 sizeof(sin.sin_addr) );
+	sin.sin_port = sin6->sin6_port;
+	memcpy ( &cs, &sin, sizeof(sin) );
+    }
+#endif /* INET6 */
+
     /*
      * Save the dotted decimal form of the client's IP address 
      * in the POP parameter block 
      */
-    p->ipaddr = (char *) strdup ( inet_ntoa ( cs.sin_addr ) );
+    p->ipaddr = (char *) strdup ( sock_ntop ( (struct sockaddr *)&cs, len ) );
 
     /*  
      * Save the client's port 
      */
-    p->ipport = ntohs ( cs.sin_port );
+    p->ipport = sock_port ( (struct sockaddr *)&cs, len );
 
     if ( no_rev_lookup )
         p->client = p->ipaddr;
@@ -1415,6 +1476,16 @@ void get_client_info ( POP *p, BOOL no_rev_lookup )
         /*  
          * Get the canonical name of the host to whom I am speaking 
          */
+#ifdef INET6
+	error = getnameinfo ( (struct sockaddr *)&cs, len,
+			      hbuf, sizeof(hbuf), NULL, 0, NI_NAMEREQD );
+        if ( error ) {
+            msg ( HERE,
+                  "(v%s) Unable to get canonical name of client %s: %s (%d)",
+                  VERSION, p->ipaddr, gai_strerror(error), error );
+            p->client = p->ipaddr;
+        }
+#else /* INET6 */
         ch = gethostbyaddr ( (char *) &cs.sin_addr, sizeof(cs.sin_addr), AF_INET );
         if ( ch == NULL ) {
             msg ( HERE,
@@ -1422,12 +1493,68 @@ void get_client_info ( POP *p, BOOL no_rev_lookup )
                   VERSION, p->ipaddr, hstrerror(h_errno), h_errno );
             p->client = p->ipaddr;
         }
+#endif /* INET6 */
         /*  
          * Save the cannonical name of the client host in 
          * the POP parameter block 
          */
         else {
 
+#ifdef INET6
+#ifndef BIND43
+            p->client = (char *) strdup ( hbuf );
+#else
+
+	    char h_name[MAXHOSTNAMELEN + 1];
+
+	    strncpy(h_name, hbuf, sizeof(h_name));
+
+	    /*  See if the name obtained for the client's IP 
+		address returns an address */
+	    memset ( &hints, 0, sizeof ( hints ) );
+	    hints.ai_family = ((struct sockaddr *)&cs)->sa_family;
+	    hints.ai_socktype = SOCK_STREAM;
+	    hints.ai_flags = AI_PASSIVE | AI_CANONNAME;
+	    res = NULL;
+	    error =  getaddrinfo ( h_name, NULL, &hints, &res );
+	    if ( error || res->ai_canonname == NULL ) {
+		msg ( HERE,
+		      "Client at \"%s\" resolves to an unknown host name \"%s\"",
+		      p->ipaddr, h_name );
+		p->client = p->ipaddr;
+	    }
+	    else {
+		/*  Save the host name (the previous value was 
+		    destroyed by gethostbyname) */
+		p->client = (char *)strdup(res->ai_canonname);
+
+		/*  Look for the client's IP address in the list returned 
+		    for its name */
+		for ( r = res; r; r = r->ai_next ) {
+		    int niflags = NI_NUMERICHOST;
+#ifdef NI_WITHSCOPEID
+		    if (r->ai_addr->sa_family == AF_INET6)
+			niflags |= NI_WITHSCOPEID;
+#endif
+		    /* We don't use sock_ntop() here due to avoid matching
+		     * error wrongly */
+		    error = getnameinfo ( r->ai_addr, r->ai_addrlen,
+					  hbuf, sizeof(hbuf), NULL, 0,
+					  niflags );
+		    if ( !error && strcmp ( hbuf, p->ipaddr ) == 0 ) break;
+		}
+	    }
+	    if ( res )
+		freeaddrinfo ( res );
+	    if ( !r ) {
+		msg ( HERE,
+		      "Client address \"%s\" not listed for its host name \"%s\"",
+		      p->ipaddr, h_name );
+		p->client = p->ipaddr;
+	    }
+
+#endif /* BIND43 */
+#else /* INET6 */
 #  ifndef    BIND43
             p->client = (char *) strdup ( ch->h_name );
 #  else /* using BIND43 */
@@ -1494,6 +1621,7 @@ void get_client_info ( POP *p, BOOL no_rev_lookup )
 #    endif /* RES_DEFNAMES */
 
 #  endif /* BIND43 */
+#endif /* INET6 */
                 }
 
     } /* no_rev_lookup */
@@ -1501,6 +1629,45 @@ void get_client_info ( POP *p, BOOL no_rev_lookup )
 #endif /* _DEBUG */
 }
 
+char *
+sock_ntop ( p, salen )
+    struct sockaddr *p;
+    int salen;
+{
+#ifdef INET6
+    static char hbuf[NI_MAXHOST];
+    int niflags;
+
+    niflags = NI_NUMERICHOST;
+#ifdef NI_WITHSCOPEID
+    if ( ((struct sockaddr *)p)->sa_family == AF_INET6 )
+	niflags |= NI_WITHSCOPEID;
+#endif
+    if ( getnameinfo ( p, salen, hbuf, sizeof(hbuf), NULL, 0, niflags ) )
+	return "(invalid)";
+    return hbuf;
+#else /* INET6 */
+    return inet_ntoa ( ((struct sockaddr_in *)p)->sin_addr );
+#endif /* INET6 */
+}
+
+sock_port ( p, salen )
+    struct sockaddr *p;
+    int salen;
+{
+#ifdef INET6
+    static char pbuf[NI_MAXSERV];
+
+    if ( getnameinfo ( p, salen, NULL, 0, pbuf, sizeof(pbuf),
+		       NI_NUMERICSERV ) == 0 )
+	return atoi ( pbuf );
+    else
+	return 0;
+#else /* INET6 */
+    return ntohs ( ((struct sockaddr_in *)p)->sin_port );
+#endif /* INET6 */
+}
+
 
 /*--------------------------------------------------------------
  * Copies a string, converting non-printables into codes.
