@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
@@ -33,11 +33,15 @@ drac_it ( POP *p )
 {
     char     *err         = NULL;
     int       rslt        = 0;
+    uint32_t  inaddr;
 
 
     if ( p->drac_host != NULL && *p->drac_host != '\0' )   
     {
-        rslt = dracauth ( p->drac_host, inet_addr(p->ipaddr), &err );
+	/* DRAC doesn't support IPv6 */
+	if ( !inet_pton ( AF_INET, p->ipaddr, &inaddr ) )
+            return POP_SUCCESS;
+        rslt = dracauth ( p->drac_host, inaddr, &err );
         if ( rslt != 0 )
             pop_log ( p, POP_PRIORITY, HERE, 
                       "[drac]: dracauth returned %d: %s",
