@@ -1,10 +1,14 @@
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
  * Revisions: 
  *
+ * 11/16/01  [secabeen]
+ *         - Added support for putting .cache and .pop files in 
+ *           home directories
+ *
  * 03/26/01  [rcg]
  *         - Fixed double-slash in path when hash-spool=1.
  *
@@ -133,7 +137,10 @@ genpath ( POP *p, char *pszDrop, int iDropLen, GNPH_WHICH iWhich )
             break;
 
         case GNPH_POP:    /* .pop file  */
-            len1 = strlcpy ( pszDrop, p->pCfg_temp_dir, iDropLen );
+            if ( p->bHome_dir_misc == TRUE )
+                len1 = strlcpy ( pszDrop, pw->pw_dir,        iDropLen );
+            else
+                len1 = strlcpy ( pszDrop, p->pCfg_temp_dir,  iDropLen );
             len2 = strlcat ( pszDrop, "/",              iDropLen );
             break;
 
@@ -143,7 +150,10 @@ genpath ( POP *p, char *pszDrop, int iDropLen, GNPH_WHICH iWhich )
             break;
 
         case GNPH_CACHE:   /* .cache file  */
-            len1 = strlcpy ( pszDrop, p->pCfg_cache_dir, iDropLen );
+            if ( p->bHome_dir_misc == TRUE )
+                len1 = strlcpy ( pszDrop, pw->pw_dir,        iDropLen );
+            else
+                len1 = strlcpy ( pszDrop, p->pCfg_cache_dir, iDropLen );
             len2 = strlcat ( pszDrop, "/",               iDropLen );
             break;
 
