@@ -1,13 +1,25 @@
 /* 
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file license.txt specifies the terms for use, modification,
  * and redistribution.
  *
  * Revisions:
- *
  *    07/12/09   [rg]
  *              - Stop using sys_nerr and sys_errlist.
  *
+ *    12/04/07   [rg]
+ *              - Updated IPv6 patch (to check for socklen_t)
+ *                from Hajimu UMEMOTO,
+ *
+ *    09/19/06   [rcg]
+ *              - Updated IPv6 patch from Hajimu UMEMOTO
+ *
+ *    08/08/06   [rcg]
+ *              - IPv6 patch by Hajimu UMEMOTO
+ *
+ *    03/09/06   [rcg]
+ *              - Now setting SO_KEEPALIVE on socket.
+ *
  *     04/25/05  [rcg]
  *              - Fix '-V' for standalone.
  *
@@ -133,6 +145,27 @@
 #  endif /* HAVE_PAM_PAM_APPL_H */
 #endif /* USE_PAM */
 
+#ifndef   HAVE_SOCKLEN_T
+typedef unsigned int socklen_t;
+#endif /* HAVE_SOCKLEN_T */
+
+#ifdef    INET6
+#  include <netdb.h>
+#else /* not INET6 */
+#  ifndef HAVE_GETADDRINFO
+struct addrinfo {
+	int	ai_flags;
+	int	ai_family;
+	int	ai_socktype;
+	int	ai_protocol;
+	socklen_t ai_addrlen;
+	char	*ai_canonname;
+	struct	sockaddr *ai_addr;
+	struct	addrinfo *ai_next;
+};
+#  endif /* not HAVE_GETADDRINFO */
+#endif /* INET6 */
+
 #ifndef  STANDALONE
 
 /*
@@ -174,11 +207,19 @@ main ( int argc, char *argv[] )
 #else /* STANDALONE */
 
 
-#ifdef _DEBUG
-#  define SERV_TCP_PORT 8765
+#ifdef    INET6
+#  ifdef _DEBUG
+#    define SERV_TCP_PORT "8765"
+#  else
+#    define SERV_TCP_PORT  "110"
+#  endif /* _DEBUG */
 #else
-#  define SERV_TCP_PORT  110
-#endif /* _DEBUG */
+#  ifdef _DEBUG
+#    define SERV_TCP_PORT 8765
+#  else
+#    define SERV_TCP_PORT  110
+#  endif /* _DEBUG */
+#endif /* INET6 */
 
 #define BAD_ADDR ( (unsigned long) -1 )
 
@@ -209,7 +250,14 @@ int     reaper  ( SIGPARAM );
 int     hupit   ( SIGPARAM );
 int     cleanup ( SIGPARAM );
 void    roll_it ( void );
-void    motherforker ( int newsockfd, int sockfd );
+void    motherforker ( int newsockfd, int pollfd[], int npollfd );
+
+
+/*
+ * Prototypes from other Qpopper files
+ */
+char * sock_ntop ( struct sockaddr *p, int salen );
+int    sock_port ( struct sockaddr *p, int salen );
 
 
 /*
@@ -241,11 +289,24 @@ main ( int argc, char *argv[] )
     int                 clilen      =  0;
     int                 i           =  0;
     int                 rslt        =  0;
+    char               *ptr         = NULL;
+    struct addrinfo     hints, *res, *r;
+#ifdef    INET6
+    struct sockaddr_storage cli_addr;
+    char               *port        = SERV_TCP_PORT;
+    char               *addr        = NULL;
+    int                 error;
+    char                b [ NI_MAXHOST ] = "";
+#else
     struct sockaddr_in  cli_addr;
     struct sockaddr_in  serv_addr;
-    char               *ptr         = NULL;
     unsigned short      port        = SERV_TCP_PORT;
     unsigned long       addr        = INADDR_ANY;
+#endif /* INET6 */
+    int                *pollfd;
+    int                 npollfd     =  0;
+    int                 maxfd       =  -1;
+    fd_set              defreadfds, readfds;
     fd_set              fdset_templ;
     fd_set              fdset_read;
     int                 fd_flags    = 0;
@@ -279,11 +340,13 @@ main ( int argc, char *argv[] )
 
     err_out = msg_out  = fopen ( "/dev/null", "w+" ); /* until we get set up */
 
+#ifndef INET6
     /*
      * Ensure default port & address is in network order
      */
     addr = htonl ( addr );
     port = htons ( port );
+#endif /* not INET6 */
 
     /*
      * Set defaults for Qargc and Qargv
@@ -308,12 +371,18 @@ main ( int argc, char *argv[] )
      * to.  Otherwise we use defaults.
      */
     ptr = argv [ 1 ];
-    if ( argc >= 2 && ( *ptr == ':' || isdigit ( (int) *ptr ) ) )
+    if ( argc >= 2 && ( *ptr == ':' ||
+#ifdef    INET6
+			*ptr == '[' ||
+#endif /* INET6 */
+			isdigit ( (int) *ptr ) ) )
     {
+#ifndef INET6
         int            j = 0;
         unsigned long  a = addr;
         unsigned short n = port;
         char           b [ 25 ] = "";
+#endif /* not INET6 */
         char          *q = b;
 
         /*
@@ -322,10 +391,20 @@ main ( int argc, char *argv[] )
         if ( strchr ( ptr, '.' ) != NULL )
             while ( *ptr == '.' || isdigit ( (int) *ptr ) )
                 *q++ = *ptr++;
+#ifdef    INET6
+    else if ( *ptr == '[' && strchr ( ptr + 1, ']' ) != NULL )
+    {
+        ptr++;
+        while ( *ptr != '\0' && *ptr != ']' )
+                *q++ = *ptr++;
+    }
+#endif /* INET6 */
         
         if ( *b != '\0' )
         {
+#ifndef INET6
             a   = inet_addr ( b );
+#endif /* not INET6 */
             ptr = strchr ( ptr, ':' );
             if ( ptr != NULL )
                 ptr++;
@@ -337,6 +416,19 @@ main ( int argc, char *argv[] )
                 ptr++;
         }
 
+#ifdef    INET6
+        if ( *b != '\0' )
+            addr = b;
+        if ( ptr != NULL )
+            port = ptr;
+        memset ( &hints, 0, sizeof(hints) );
+        hints.ai_flags = AI_PASSIVE | AI_NUMERICHOST;
+        hints.ai_family = PF_UNSPEC;
+        hints.ai_socktype = SOCK_STREAM;
+        if ( getaddrinfo ( addr, port, &hints, &res ) != 0 )
+                err_dump ( HERE, "invalid address and/or port: \"%s\"", argv[1] );
+        freeaddrinfo ( res );
+#else /* not INET6 */
         /*
          * We might have a port number
          */
@@ -349,6 +441,7 @@ main ( int argc, char *argv[] )
 
         port = htons ( n );
         addr = a;
+#endif
         
         /*
          * Since we consumed the first specified parameter,
@@ -562,92 +655,164 @@ main ( int argc, char *argv[] )
 
 #endif /* not _DEBUG */
 
-    /*
-     * Set up the socket on which we listen
-     */
-    sockfd = socket ( AF_INET, SOCK_STREAM, 0 );
-    if ( sockfd < 0 )
-        err_dump ( HERE, "Can't open stream socket" );
-    TRACE ( trace_file, POP_DEBUG, HERE, 
-            "opened stream socket; sockfd = %d", 
-            sockfd );
- 
-    i = 1;
-    rslt = setsockopt ( sockfd, SOL_SOCKET, SO_REUSEADDR, 
-                        (char *) &i, sizeof(i) );
-    if ( rslt == -1 )
-        err_dump ( HERE, "setsockopt(SO_REUSEADDR) failed" );
-
-    if ( debug )
+    memset ( &hints, 0, sizeof(hints) );
+    hints.ai_socktype = SOCK_STREAM;
+#ifdef INET6
+    hints.ai_flags = AI_PASSIVE;
+    hints.ai_family = PF_UNSPEC;
+    error = getaddrinfo ( addr, port, &hints, &res );
+    if ( error )
     {
-        rslt = setsockopt ( sockfd, SOL_SOCKET, SO_DEBUG,
-                            (char *) &i, sizeof(i) );
-        if ( rslt == -1 )
-            TRACE ( trace_file, POP_DEBUG, HERE, 
-                    "%s: Server: setsockopt(SO_DEBUG) failed", pname );
+        fprintf ( stderr, "%s\n", gai_strerror(error) );
+        return 1;
     }
-
-    TRACE ( trace_file, POP_DEBUG, HERE, 
-            "set stream socket options; sockfd = %d", 
-            sockfd );
-
-    bzero ( (char *) &serv_addr, sizeof(serv_addr) );
+#else /* not INET6 */
+    memset ( &serv_addr, 0, sizeof(serv_addr) );
     serv_addr.sin_family      = AF_INET;
     serv_addr.sin_addr.s_addr = addr;
     serv_addr.sin_port        = port;
+    hints.ai_addr = (struct sockaddr *)&serv_addr;
+    hints.ai_addrlen = sizeof(serv_addr);
+    hints.ai_family = serv_addr.sin_family;
+    res = &hints;
+#endif /* INET6 */
 
-    rslt = bind ( sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr) );
-    if ( rslt < 0 )
+    /* Count max number of sockets we may open */
+    for ( npollfd = 0, r = res; r; r = r->ai_next, npollfd++ )
+        ;
+
+    pollfd = malloc ( npollfd * sizeof(int) );
+    if ( !pollfd )
     {
-        if ( errno == EADDRINUSE )
-        {
-            fprintf ( stderr, "%s:%d in use\n",
-                      inet_ntoa ( serv_addr.sin_addr ),
-                      ntohs     ( serv_addr.sin_port ) );
-            return 1;
-        }
-        else
-            err_dump ( HERE, "Can't bind local address %s:%d",
-                       inet_ntoa ( serv_addr.sin_addr ),
-                       ntohs     ( serv_addr.sin_port ) );
+        fprintf ( stderr, "couldn't allocate memory for sockets\n" );
+        return 1;
     }
 
-    TRACE ( trace_file, POP_DEBUG, HERE,
-            "did bind on stream socket; sockfd = %d",
-            sockfd );
-
-
-    /*
-     * Now we're ready to go
-     */
-    msg ( HERE, "listening on %s:%d\n",
-          inet_ntoa ( serv_addr.sin_addr ),
-          ntohs     ( serv_addr.sin_port ) );
-
-    TRACE ( trace_file, POP_DEBUG, HERE, 
-            "listening using socket fd %d", 
-            sockfd );
-
-    listen ( sockfd, 5 );
-
-    /*
-     * Set file descriptor to be non-blocking in case there isn't really a
-     * connection available if the select succeeds.  This avoids us
-     * blocking there.
-     */
-    fd_flags = fcntl ( sockfd, F_GETFL, 0 );
-    rslt     = fcntl ( sockfd, F_SETFL, fd_flags | O_NONBLOCK );
-    if ( rslt == -1 )
-        err_dump ( HERE, "Unable to set sockfd(%d) to be non-blocking",
-                   sockfd );
-    TRACE ( trace_file, POP_DEBUG, HERE, "set fd %d non-blocking (%#x)",
-            sockfd, fcntl ( sockfd, F_GETFL, 0 ) );
-
-    /*
-     * Build the template descriptor set
-     */
     memset ( &fdset_templ, 0, sizeof(fdset_templ) );
-    FD_SET ( sockfd, &fdset_templ );
+    npollfd = 0;   /* num of sockets counter at start of array */
+    i = 1;
+    for ( r = res; r; r = r->ai_next )
+    {
+        /*
+         * Set up the socket on which we listen
+         */
+        sockfd = socket ( r->ai_family, r->ai_socktype, r->ai_protocol );
+        if ( sockfd < 0 )
+            continue;
+        TRACE ( trace_file, POP_DEBUG, HERE,
+                "opened stream socket; sockfd = %d", sockfd );
+
+        rslt = setsockopt ( sockfd, SOL_SOCKET, SO_REUSEADDR,
+                            (char *) &i, sizeof(i) );
+        if ( rslt == -1 )
+        {
+            TRACE ( trace_file, POP_DEBUG, HERE, 
+                    "%s: Server: setsockopt(SO_REUSEADDR) failed", pname );
+            close ( sockfd );
+            continue;
+        }
+
+        rslt = setsockopt ( sockfd, SOL_SOCKET, SO_KEEPALIVE,
+                            (char *) &i, sizeof(i) );
+        if ( rslt == -1 )
+        {
+            TRACE ( trace_file, POP_DEBUG, HERE,
+                    "%s: Server: setsockopt(SO_KEEPALIVE) failed", pname );
+            close ( sockfd );
+            continue;
+        }
+
+#ifdef    IPV6_V6ONLY
+        if ( r->ai_family == AF_INET6 )
+        {
+            rslt = setsockopt ( sockfd, IPPROTO_IPV6, IPV6_V6ONLY,
+                                (char *) &i, sizeof(i) );
+            if ( rslt == -1 )
+            {
+                TRACE ( trace_file, POP_DEBUG, HERE, 
+                        "%s: Server: setsockopt(IPV6_V6ONLY) failed",
+                        pname );
+                close ( sockfd );
+                continue;
+            }
+        }
+#endif /* IPV6_V6ONLY */
+
+        if ( debug )
+        {
+            rslt = setsockopt ( sockfd, SOL_SOCKET, SO_DEBUG,
+                                (char *) &i, sizeof(i) );
+            if ( rslt == -1 )
+                TRACE ( trace_file, POP_DEBUG, HERE, 
+                        "%s: Server: setsockopt(SO_DEBUG) failed", pname );
+        }
+
+        TRACE ( trace_file, POP_DEBUG, HERE,
+                "set stream socket options; sockfd = %d", sockfd );
+
+        rslt = bind ( sockfd, r->ai_addr, r->ai_addrlen );
+        if ( rslt < 0 )
+        {
+            if ( errno == EADDRINUSE )
+                fprintf ( stderr, "%s:%d in use\n",
+                          sock_ntop ( r->ai_addr, r->ai_addrlen ),
+                          sock_port ( r->ai_addr, r->ai_addrlen ) );
+            else
+                fprintf ( stderr, "Can't bind local address %s:%d",
+                          sock_ntop ( r->ai_addr, r->ai_addrlen ),
+                          sock_port ( r->ai_addr, r->ai_addrlen ) );
+            close ( sockfd );
+            continue;
+        }
+
+        TRACE ( trace_file, POP_DEBUG, HERE,
+                "did bind on stream socket; sockfd = %d",
+                sockfd );
+
+        /*
+         * Now we're ready to go
+         */
+        msg ( HERE,
+              ( r->ai_family == AF_INET6 ) ? "listening on [%s]:%d\n" :
+                                             "listening on %s:%d\n",
+              sock_ntop ( r->ai_addr, r->ai_addrlen ),
+              sock_port ( r->ai_addr, r->ai_addrlen ) );
+
+        TRACE ( trace_file, POP_DEBUG, HERE, "listening using socket fd %d",
+                sockfd );
+
+        listen ( sockfd, 5 );
+
+        /*
+         * Set file descriptor to be non-blocking in case there isn't really a
+         * connection available if the select succeeds.  This avoids us
+         * blocking there.
+         */
+        fd_flags = fcntl ( sockfd, F_GETFL, 0 );
+        rslt     = fcntl ( sockfd, F_SETFL, fd_flags | O_NONBLOCK );
+        if ( rslt == -1 )
+            err_dump ( HERE, "Unable to set sockfd(%d) to be non-blocking",
+                       sockfd );
+        TRACE ( trace_file, POP_DEBUG, HERE, "set fd %d non-blocking (%#x)",
+                sockfd, fcntl ( sockfd, F_GETFL, 0 ) );
+
+        /*
+         * Build the template descriptor set
+         */
+        FD_SET ( sockfd, &fdset_templ );
+
+        if ( maxfd < sockfd )
+            maxfd = sockfd;
+        pollfd[npollfd++] = sockfd;
+    }
+
+#ifdef INET6
+    if ( res )
+        freeaddrinfo ( res );
+#endif /* INET6 */
+
+    if ( npollfd == 0 )
+        err_dump ( HERE, "Can't bind to any socket" );
 
 #ifdef    BSD
     signal ( SIGCHLD, VOIDSTAR reaper  );
@@ -663,8 +828,10 @@ main ( int argc, char *argv[] )
         if ( bClean )
         {
             msg   ( HERE, "cleaning up and exiting normally" );
-            close ( sockfd );
-            sockfd = -1;
+            for ( i = 0; i < npollfd; i++ ) {
+                close ( pollfd[i] );
+                pollfd[i] = -1;
+            }
             if ( trace_file != NULL )
             {
                 fclose ( trace_file );
@@ -690,29 +857,36 @@ main ( int argc, char *argv[] )
          * Check for a new connection with select() before calling accept(),
          * since accept() does not return on signals on some platforms.
          */
-        rslt = select ( sockfd + 1, &fdset_read, NULL, NULL, NULL );
+        rslt = select ( maxfd + 1, &fdset_read, NULL, NULL, NULL );
         if ( rslt == -1 && errno != EINTR )
             err_dump ( HERE, "select() error" );
 
-        if ( rslt == 0 || ( FD_ISSET(sockfd, &fdset_read) == FALSE ) )
+        if ( rslt == 0 )
             err_dump ( HERE, "unexpected select() result" ); 
 
-        clilen    = sizeof(cli_addr);
-        newsockfd = accept ( sockfd, (struct sockaddr *) &cli_addr, &clilen );
+        for ( i = 0; i < npollfd; ++i )
+        {
+            if ( !FD_ISSET ( pollfd[i], &fdset_read ) )
+                continue;
 
-        TRACE ( trace_file, POP_DEBUG, HERE, 
-                "accept=%d; sockfd=%d; clilen=%d; cli_addr=%s:%d\n",
-                newsockfd, sockfd, clilen, 
-                inet_ntoa ( cli_addr.sin_addr ),
-                ntohs     ( cli_addr.sin_port ) );
+            clilen    = sizeof(cli_addr);
+            newsockfd = accept ( pollfd[i], (struct sockaddr *) &cli_addr,
+                                 &clilen );
 
-        if ( newsockfd > 0 )
-            motherforker ( newsockfd, sockfd ); 
-        else
-            if ( errno != EINTR && errno != EWOULDBLOCK 
-                                && errno != EPROTO
-                                && errno != ECONNABORTED )
-                err_msg ( HERE, "accept() error" );
+            TRACE ( trace_file, POP_DEBUG, HERE, 
+                    "accept=%d; sockfd=%d; clilen=%d; cli_addr=%s:%d\n",
+                    newsockfd, pollfd[i], clilen, 
+                    sock_ntop ( (struct sockaddr *) &cli_addr, clilen ),
+                    sock_port ( (struct sockaddr *) &cli_addr, clilen ) );
+
+            if ( newsockfd > 0 )
+                motherforker ( newsockfd, pollfd, npollfd ); 
+            else
+                if ( errno != EINTR && errno != EWOULDBLOCK 
+                                    && errno != EPROTO
+                                    && errno != ECONNABORTED )
+                    err_msg ( HERE, "accept() error" );
+        }
 
     } /* main loop */
 
@@ -728,9 +902,9 @@ msg ( WHENCE, const char *format, ... )
 {
     va_list  ap;                       /* Pointer into stack to extract
                                         *     parameters */
-    size_t   left   = sizeof(msg_buf);
-    size_t   len    = 0;
-    int      iChunk = 0;
+    int   left   = sizeof(msg_buf);
+    int   len    = 0;
+    int   iChunk = 0;
 
 
     va_start ( ap, format );
@@ -766,9 +940,9 @@ err_dump ( WHENCE, const char *format, ... )
 {
     va_list  ap;                       /* Pointer into stack to extract
                                         *     parameters */
-    size_t   left   = sizeof(msg_buf);
-    size_t   len    = 0;
-    int      iChunk = 0;
+    int   left   = sizeof(msg_buf);
+    int   len    = 0;
+    int   iChunk = 0;
 
 
     va_start ( ap, format );
@@ -788,7 +962,7 @@ err_dump ( WHENCE, const char *format, ... )
     left  -= ( iChunk > 0 ? iChunk : left );
     len   += ( iChunk > 0 ? iChunk : left );
 
-    iChunk = Qsnprintf ( msg_buf + len, left, ": %s [%s:%lu]",
+    iChunk = Qsnprintf ( msg_buf + len, left, ": %s [%s:%zu]",
                          sys_err_str(), fn, ln );
     left  -= ( iChunk > 0 ? iChunk : left );
     len   += ( iChunk > 0 ? iChunk : left );
@@ -826,9 +1000,9 @@ err_msg ( WHENCE, const char *format, ... )
     va_list  ap;                       /* Pointer into stack to extract
                                         *     parameters */
 
-    size_t   left   = sizeof(msg_buf);
-    size_t   len    = 0;
-    int      iChunk = 0;
+    int   left   = sizeof(msg_buf);
+    int   len    = 0;
+    int   iChunk = 0;
 
 
     va_start ( ap, format );
@@ -850,7 +1024,7 @@ err_msg ( WHENCE, const char *format, ... )
 
     va_end   ( ap );
 
-    iChunk = Qsnprintf ( msg_buf + len, left, ": %s [%s:%lu]",
+    iChunk = Qsnprintf ( msg_buf + len, left, ": %s [%s:%zu]",
                          sys_err_str(), fn, ln );
     left  -= ( iChunk > 0 ? iChunk : left );
     len   += ( iChunk > 0 ? iChunk : left );
@@ -980,11 +1154,12 @@ roll_it ( void )
  * Handles new client connection
  */
 void
-motherforker ( int newsockfd, int sockfd )
+motherforker ( int newsockfd, int pollfd[], int npollfd )
 {
     int     childpid    = 0;
     int     fd_flags    = 0;
     int     rslt        = 0;
+    int     i;
 
 
     TRACE ( trace_file, POP_DEBUG, HERE, "new connection; fd=%d", newsockfd );
@@ -1023,8 +1198,10 @@ motherforker ( int newsockfd, int sockfd )
         /*
          * We don't need sockfd
          */
-        close ( sockfd );
-        sockfd = -1;
+	for ( i = 0; i < npollfd; ++i ) {
+	    close ( pollfd[i] );
+	    pollfd[i] = -1;
+	}
 
 #endif /* not _DEBUG */
 
@@ -1036,8 +1213,10 @@ motherforker ( int newsockfd, int sockfd )
         qpopper ( Qargc, Qargv );
             
 #ifdef _DEBUG
-        close  ( sockfd );
-        sockfd = -1;
+	for ( i = 0; i < npollfd; ++i ) {
+	    close ( pollfd[i] );
+	    pollfd[i] = -1;
+	}
 #endif /* not _DEBUG */
 
         TRACE ( trace_file, POP_DEBUG, HERE, "exiting after Qpopper returned" );
