@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2005 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The license.txt file specifies the terms for use, 
  * modification, and redistribution.
  *
@@ -190,17 +190,17 @@ POP *p;
         p->bNo_mime     = TRUE;
         user_name_len  -= 8;
         DEBUG_LOG4 ( p, "Turned on MIME mangling; "
-                        "user name entered was (%lu)'%.100s'; "
-                        "user name reset to (%u)'%.100s'",
+                        "user name entered was (%zu)'%.100s'; "
+                        "user name reset to (%zu)'%.100s'",
                      strlen(p->pop_parm[1]),   p->pop_parm[1],
-                     (unsigned) user_name_len, p->pop_parm[1] );
+                     user_name_len, p->pop_parm[1] );
       }
     
     if ( user_name_len > ( sizeof(p->user) -1 ) ) {
-        pop_log ( p, POP_WARNING, HERE, "Excessive user name length (%lu); "
-                                  "truncated to %lu: %.100s",
-                  (long unsigned) user_name_len, 
-                  (long unsigned) (sizeof(p->user) -1), 
+        pop_log ( p, POP_WARNING, HERE, "Excessive user name length (%zu); "
+                                  "truncated to %zu: %.100s",
+                  user_name_len, 
+                  (sizeof(p->user) -1), 
                   p->pop_parm[1] );
         user_name_len = sizeof(p->user) -1;
     }
@@ -229,7 +229,7 @@ POP *p;
     if ( pw != NULL ) {
         p->pw        = *pw;                  /* copy it */
         p->pw.pw_dir = strdup(pw->pw_dir);   /* copy home directory */
-        DEBUG_LOG2 ( p, "home (%lu): '%s'",
+        DEBUG_LOG2 ( p, "home (%zu): '%s'",
                      strlen(p->pw.pw_dir), p->pw.pw_dir );
     }
     if ( ( pw == NULL ) || (  pw->pw_passwd == NULL )
@@ -285,11 +285,11 @@ POP *p;
          && ( ( st.st_mode & 0777 ) != 0600 ) ) {
         dbm_close ( db );
         DEBUG_LOG4 ( p, "stat returned %d; %s mode is %o (%x)",
-                     j, apop_dir, st.st_mode, st.st_mode );
+                     j, apop_dir, (int) st.st_mode, (int) st.st_mode );
         return ( pop_auth_fail ( p, POP_FAILURE, HERE,
                                  "[SYS/PERM] POP authentication DB has "
                                  "wrong mode (%.3o)",
-                                 st.st_mode & 0777 ) );
+                                 (int) st.st_mode & 0777 ) );
     }
     f = open ( apop_dir, O_RDONLY );
     if ( f == -1 ) {
