@@ -2,13 +2,15 @@
  * SASL authentication mechanisms for qpopper
  *
  * File:      pop_auth.c
- * Copyright: (c) 2003 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright: (c) 2001 QUALCOMM Incorporated.  All rights reserved.
  *            The file license.txt specifies the terms for use,
  *            modification, and redistribution.
  */
 
 /*
  * Revisions:
+ *     06/21/08  [rcg]
+ *             - Use new "uint4.h" for UINT4 typedef
  *
  *     08/13/03  [rcg]
  *             - Patch from Uli Zappe to fix SCRAM compilation bugs.
@@ -64,6 +66,7 @@
 #include <stdio.h>
 
 #include "config.h"
+#include "uint4.h"
 
 #if HAVE_FCNTL_H
 #  include <fcntl.h>
@@ -105,6 +108,10 @@ static struct passwd    * pw;
 
 #endif  /* ifdef SCRAM */
 
+#if defined(SCRAM) && defined(CYRUS_SASL)
+#  error "Only one of SCRAM or CYRUS_SASL may be defined"
+#endif /* SCRAM && CYRUS_SASL */
+
 extern char    *ERRMSG_PW;
 
 static auth_table mechanisms[] = {
@@ -116,6 +123,7 @@ static auth_table mechanisms[] = {
 
     /*  Process the Auth Command Initiation */
 
+#ifndef CYRUS_SASL
 int pop_auth (p)
 POP     *   p;
 {
@@ -144,6 +152,7 @@ POP     *   p;
     return(pop_msg(p, POP_FAILURE, HERE, "Unknown authentication mechanism: %.100s",
                    p->pop_parm[1]) );
 }
+#endif /* ! CYRUS_SASL */
 
 /* Do the real SCRAM Work */
 #ifdef SCRAM
