@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  */
@@ -134,6 +134,7 @@
 
 #include "popper.h"
 #include "string_util.h"
+#include "snprintf.h"
 
 
 static int sequence = 0;
@@ -352,6 +353,7 @@ pop_bull ( POP *p, struct passwd *pwp )
    long             lastBullSent;
    char             buffer [ MAXMSGLINELEN ];
    int              res                         = POP_SUCCESS;
+   int              err                         = 0;
    int              save_count;
    int              save_visible_count;
    long             save_drop_size;
@@ -616,8 +618,6 @@ pop_bull ( POP *p, struct passwd *pwp )
    if ( res == POP_SUCCESS ) {
 
 #ifdef BULLDB
-       int err = 0;
-
        bull_count.dptr  = (char *) &maxBullNumber;
        bull_count.dsize = sizeof(maxBullNumber);
        err = flock ( dbm_dirfno ( p->bull_db ), LOCK_EX );
@@ -721,13 +721,13 @@ CopyOneBull ( POP *p, long bnum, char *name )
     msg_num = p->msg_count;
     p->msg_count = ( ( ( p->msg_count - 1 ) / ALLOC_MSGS ) + 1 ) * ALLOC_MSGS;
 
-    nbullname = Qvsnprintf ( bullName, sizeof(bullName),
-                             "%s/%s", p->bulldir, name );
+    nbullname = Qsnprintf ( bullName, sizeof(bullName),
+                            "%s/%s", p->bulldir, name );
     if ( nbullname == -1 ) {
         pop_log ( p, POP_PRIORITY, HERE,
                  "Bulletin directory + file name too long; "
-                 "directory (%d): \"%.100s\"; "
-                 "file name (%d): \"%.100s\" ", 
+                 "directory (%zu): \"%.100s\"; "
+                 "file name (%zu): \"%.100s\" ", 
                  strlen(p->bulldir), p->bulldir,
                  strlen(name), name );
        return POP_FAILURE;
