@@ -1,11 +1,14 @@
 
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
  * Revisions: 
  *
+ * 06/28/08  [rcg]
+ *         - Use cache even if new mail arrived (thanks to A'rpi).
+ *
  * 02/23/01  [rcg]
  *         - Non-existent cache file isn't an error.
  *
@@ -263,7 +266,7 @@ cache_write ( POP *p, int spool_fd, MsgInfoList *mp )
  *     POP_SUCCESS or POP_FAILURE.
  */
 int
-cache_read ( POP *p, int spool_fd, MsgInfoList **mp )
+cache_read ( POP *p, FILE* spool_fd, MsgInfoList **mp )
 {
     char         cache_name [ MAXFILENAME ];
     cache_hdr    header;
@@ -321,56 +324,62 @@ cache_read ( POP *p, int spool_fd, MsgInfoList **mp )
     
     if ( header.toc_str_sz != sizeof ( MsgInfoList ) ) {
         pop_log ( p, POP_NOTICE, HERE,
-                  "toc size mismatch in .cache file %s: %lu vs %lu",
+                  "toc size mismatch in .cache file %s: %ld vs %zu",
                   cache_name, header.toc_str_sz, sizeof ( MsgInfoList ) );
         close  ( cache_fd );
         return POP_FAILURE;
     }
 
-    rslt = fstat ( spool_fd, &stbuf );
+    rslt = fstat ( fileno(spool_fd), &stbuf );
     if ( rslt < 0 ) {
         pop_log ( p, POP_PRIORITY, HERE,
                   "fstat() failed for spool (%d) %s (%d)",
-                  spool_fd, STRERROR(errno), errno );
+                  fileno(spool_fd), STRERROR(errno), errno );
         close  ( cache_fd );
         return POP_FAILURE;
     }
 
     if ( header.modtime > stbuf.st_mtime ) {
-        pop_log ( p, POP_PRIORITY, HERE,
+        pop_log ( p, POP_DEBUG, HERE,
                   "spool older than cache file %s",
                   cache_name );
         close ( cache_fd );
         return POP_FAILURE;
     }
 
+    /*
+     * Exit if the spool is smaller than indicated in the cache, since
+     * otherwise we'll have corruption.
+     */
     if ( header.spool_end > stbuf.st_size ) {
-        pop_log ( p, POP_PRIORITY, HERE,
-                  "spool smaller than indicated in cache file %s "
-                  "(expected %ld; found %ld)",
-                  cache_name, (long) header.spool_end, (long) stbuf.st_size );
+        pop_log ( p, POP_NOTICE, HERE,
+                  "spool smaller than indicated in "
+                  "cache: %s (size %ld modtime %ld vs %ld %ld)",
+                  cache_name, (long) header.spool_end, header.modtime,
+                  (long) stbuf.st_size, stbuf.st_mtime );
         close ( cache_fd );
         return POP_FAILURE;
     }
 
-    /*
-     * For now, exit if the spool has been modified or is larger than
-     * indicated in the cache.  This is temporary.
-     */
-    if ( header.spool_end != stbuf.st_size ||
-         header.modtime   != stbuf.st_mtime ) {
-        DEBUG_LOG5 ( p, "spool modified or larger than indicated in "
-                     "cache: %s (%ld %ld vs %ld %ld)",
+    if ( header.modtime != stbuf.st_mtime ) {
+        if ( p->bCons_cache ) {
+            pop_log ( p, POP_DEBUG, HERE,
+                  "'conservative-cache' set and spool modified since "
+                  "cache creation: %s (size %ld modtime %ld vs %ld %ld)",
+                  cache_name, (long) header.spool_end, header.modtime,
+                  (long) stbuf.st_size, stbuf.st_mtime );
+            close ( cache_fd );
+            return POP_FAILURE;
+        }
+        /* just a note in the debug log in case there are any problems s*/
+        DEBUG_LOG5 ( p, "Note: spool modified since cache creation: "
+                     "%s (size %ld modtime %ld vs %ld %ld)",
                      cache_name, (long) header.spool_end, header.modtime,
                      (long) stbuf.st_size, stbuf.st_mtime );
-        close ( cache_fd );
-        return POP_FAILURE;
     }
 
-
     /*
-     * Now we know that the cache information is probably compatible
-     * with the spool.  Note that the local delivery agent may have
+     * Note that the local delivery agent may have
      * added messages to the spool since the cache was written,
      * in which case the new messages need to be processed normally.
      */
@@ -421,6 +430,12 @@ cache_read ( POP *p, int spool_fd, MsgInfoList **mp )
         p->visible_msg_count = (*mp) [ header.msg_count - 1 ].visible_num;
     }
 
+    /*
+     * Set temp drop to end of cached mail, so any new mail in temp drop will
+     * be processed normally by init_dropinfo()
+     */
+    fseek ( spool_fd, header.spool_end, SEEK_SET );
+
     DEBUG_LOG7 ( p, "Read cache file \"%s\"; msg_count=%d; toc_size=%ld; "
                     "drop_size=%ld; spool_end=%ld; first_msg_hidden=%d; "
                     "visible_msg_count=%d",
@@ -428,6 +443,27 @@ cache_read ( POP *p, int spool_fd, MsgInfoList **mp )
                  header.drop_size, (long) header.spool_end,
                  p->first_msg_hidden, p->visible_msg_count );
 
+    /*
+     * If the spool is larger than indicated in the cache, it's likely that
+     * new mail has arrived; we return an error so that pop_dropcopy() will
+     * call init_dropinfo() to process the new messages.  Note that if the
+     * spool was modified by a process other than Qpopper or a local delivery
+     * agent, specifically if the spool has anything deleted from it since
+     * the cache was created, the MsgInfoList won't match the spool, and
+     * mail will be corrupted.
+     */
+    if ( header.spool_end < stbuf.st_size ) {
+        DEBUG_LOG5 ( p, "spool larger than indicated in "
+                        "cache: %s (size %ld modtime %ld vs %ld %ld)",
+                     cache_name, (long) header.spool_end, header.modtime,
+                     (long) stbuf.st_size, stbuf.st_mtime );
+        return POP_FAILURE;
+    }
+
+    /*
+     * If all mail in temp drop is in cache, no need for init_dropinfo()
+     * to be called.
+     */
     return POP_SUCCESS;
 }
 
