@@ -1,9 +1,21 @@
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
  * Revisions:
+ *     06/29/08  [rcg]
+ *             - Add new "conservative-cache" option to not use
+ *               cache file if its modtime is earlier than the spool.
+ *
+ *     05/5/08   [DBH]
+ *             - Change the way tokens are extracted from the input line
+ *               to be able to properly handle quoted values and non-quoted
+ *               values.
+ *
+ *     04/30/08  [rcg]
+ *             - Allow some options to have negative integer values,
+ *               specifically to allow announce-expire to be -1.
  *
  *     04/11/05  [rcg]
  *             - Process user and spool config file as user, not as
@@ -14,6 +26,10 @@
  *             - Patch by David Shrimpton to always cleanup (not
  *               just in error cases)
  *
+ *     01/18/03  [kenh]
+ *             - Added SASL items.
+ *             - Restrict clear-text-password=never to APOP or SASL.
+ *
  *     08/06/02  [rcg]
  *             - Added ability to set OpenSSL options (SSL_OP_* values).
  *               Note that unlike other options, if this is set more
@@ -125,6 +141,7 @@
 
 #define COMMENT_CHAR    '#'
 #define MAX_RECURSION   100
+#define QUOTE_CHAR      '\\'
 
 
 
@@ -174,6 +191,7 @@ typedef enum
     kER_BAD_VAL,
     kER_EXP_BOOL,
     kER_EXP_INT,
+    kER_EXP_INT_NEG,
     kER_EXP_STR,
     kER_EXP_MNEM,
     kER_EXP_EOL,
@@ -265,8 +283,8 @@ typedef enum
     kSASL_NO_ANONYMOUS,     /*    (no flag)     */
     kXTND_XMIT,             /* -x               */
     kPAM_SESSION,           /*    (no flag)     */
-
-
+    kEXT_POSTAUTH_CMD,      /*    (no flag)     */
+    kCONS_CACHE,            /*    (no flag)     */
     LAST_OPT_VAL
 } opt_val_type;
 
@@ -277,7 +295,7 @@ typedef enum
 static config_table options [] =
 {
 /*    name                             type      restricted  value                */
-    { "announce-expire"              , CfgInt  , CfgResNone, kANNEXPIRE          },
+    { "announce-expire"              , CfgIntNeg,CfgResNone, kANNEXPIRE          },
     { "announce-login-delay"         , CfgInt  , CfgResNone, kANNLOGINDELAY      },
     { "auth-file"                    , CfgStr  , CfgResInit, kAUTHFILE           },
     { "auto-delete"                  , CfgBool , CfgResUser, kAUTO_DELETE        },
@@ -290,11 +308,13 @@ static config_table options [] =
     { "check-old-spool-loc"          , CfgBool , CfgResNone, kCHECK_OLD_SPOOL_LOC},
     { "check-password-expired"       , CfgBool , CfgResInit, kCHECK_PW_MAX       },
     { "chunky-writes"                , CfgMnem , CfgResNone, kCHUNKY_WRITES      },
+    { "conservative-cache"           , CfgBool , CfgResNone, kCONS_CACHE         },
     { "clear-text-password"          , CfgMnem , CfgResUser, kCLEARPASS          },
     { "config-file"                  , CfgStr  , CfgResNone, kCONFIGFILE         },
     { "debug"                        , CfgBool , CfgResNone, kDEBUG              },
     { "downcase-user"                , CfgBool , CfgResUser, kDCASEUSER          },
     { "drac-host"                    , CfgStr  , CfgResNone, kDRACHOST           },
+    { "ext-postauth-cmd"             , CfgStr  , CfgResUser, kEXT_POSTAUTH_CMD   },
     { "fast-update"                  , CfgBool , CfgResNone, kFASTUPDATE         },
     { "group-bulletins"              , CfgBool , CfgResUser, kGROUP_BULLS        },
     { "group-no-server-mode"         , CfgStr  , CfgResUser, kGRP_NO_SERV_MODE   },
@@ -314,6 +334,14 @@ static config_table options [] =
     { "nonauth-file"                 , CfgStr  , CfgResInit, kNONAUTHFILE        },
     { "old-style-uid"                , CfgBool , CfgResNone, kOLD_STYLE_UID      },
     { "reverse-lookup"               , CfgBool , CfgResUser, kREVLOOKUP          },
+    { "sasl-log-login"               , CfgStr  , CfgResUser, kSASL_LOG_LOGIN     },
+    { "sasl-min-ssf"                 , CfgInt  , CfgResUser, kSASL_MIN_SSF       },
+    { "sasl-max-ssf"                 , CfgInt  , CfgResUser, kSASL_MAX_SSF       },
+    { "sasl-no-plaintext"            , CfgBool , CfgResUser, kSASL_NO_PLAINTEXT  },
+    { "sasl-no-active"               , CfgBool , CfgResUser, kSASL_NO_ACTIVE     },
+    { "sasl-no-dictionary"           , CfgBool , CfgResUser, kSASL_NO_DICTIONARY },
+    { "sasl-forward-secrecy"         , CfgBool , CfgResUser, kSASL_FORWARD_SEC   },
+    { "sasl-no-anonymous"            , CfgBool , CfgResUser, kSASL_NO_ANONYMOUS  },
     { "server-mode"                  , CfgBool , CfgResNone, kSERVER             },
     { "shy"                          , CfgBool , CfgResInit, kSHY                },
     { "spool-dir"                    , CfgStr  , CfgResUser, kCFG_SPOOL_DIR      },
@@ -419,8 +447,13 @@ static mnemonic_map mnem_map_chunky_writes [] =
 {
     { "default"         , ChunkyAlways          },
     { "always"          , ChunkyAlways          },
+#ifdef    QPOP_SSL
     { "TLS"             , ChunkyTLS             },
     { "SSL"             , ChunkyTLS             },
+#endif /* QPOP_SSL */
+#ifdef    CYRUS_SASL
+    { "SASL"            , ChunkySASL            },
+#endif /* CYRUS_SASL */
     { "never"           , ChunkyNever           },
 
     { NULL              , ChunkyAlways          }
@@ -500,14 +533,9 @@ static void          tilt (POP *p, error_code_type er, char *token, int len, siz
 static config_table *find_option (char *token, long len);
 static int           find_mnemonic ( config_table *opt, char *token, long len );
 static char         *get_mnemonic ( config_table *opt, int val );
-static BOOL          char_is_quote         (char *token, char *buffer);
-static long          length_in_quoted_str  (char *token, long line_length);
-static long          length_to_white_space (char *token, long line_length);
-static opt_ptr_type  get_opt_ptr ( POP *p, opt_val_type item );
-static long          get_string_length (char **token, long *line_length, char *buffer);
-static long          get_token_length  (char  *token, long  line_length);
-static void          skip_white_space  (char **token, long *line_length);
-static void          skip_token (config_opt_type ttype, char **token, long *token_len, long *line_len);
+static BOOL          char_is_quote         (char *token);
+static long          get_next_token  (char  **ptok, char **ntok);
+static void          skip_white_space  (char **token);
 static void          show_result (POP *p, config_table *opt, int iVal, const char *sVal, WHENCE);
 static error_code_type handle_value (POP *p, config_table *opt, char *pval, int plen, config_call_type CallTime);
 
@@ -648,11 +676,18 @@ tilt ( POP              *p,
 
         case kER_EXP_INT:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
-                      "Integer value expected for the %s option; "
+                      "Integer value (positive) expected for the %s option; "
                       "scanning \"%.*s\" at line %lu of config file %s",
                       ctp->name, MIN ( 128, len), token, (u_long)line_no, cfile );
             break;
 
+        case kER_EXP_INT_NEG:
+            pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
+                      "Integer value (may be negative) expected for the %s "
+                      "option; scanning \"%.*s\" at line %lu of config file %s",
+                      ctp->name, MIN ( 128, len), token, (u_long)line_no, cfile );
+            break;
+
         case kER_EXP_STR:
             pop_log ( p, POP_PRIORITY, fn, ln, /* fn and ln are part of WHENCE */
                       "String value expected for the %s option; "
@@ -825,59 +860,14 @@ get_mnemonic ( config_table *opt, int val )
  * Is character a quote character?
  */
 static BOOL
-char_is_quote ( char *token, char *buffer )
+char_is_quote ( char *token)
 {
     if ( *token != '\"' && *token != '`' && *token != '\'' )
         return FALSE;
-    if ( ( token - buffer ) > 0 && *(token - 1) == '\\' )
-        return FALSE;
     return TRUE;
 }
 
 
-/*
- * Return number of characters in quoted string.
- */
-static long
-length_in_quoted_str ( char *token, long line_length )
-{
-    char *orig_ptr = token;
-    char  qchar    = *token;
-
-
-    if ( qchar != '\"' && qchar != '`' && qchar != '\'' )
-        return 0;
-
-    token++; /* skip past open quote */
-    line_length--;
-    while ( line_length > 0 && ( *token != qchar || *(token - 1) == '\\' )  )
-    {
-        line_length--;
-        token++;
-    }
-    
-    return ( token - orig_ptr - 1 );
-}
-
-
-/*
- * Return number of characters until white space.
- */
-static long
-length_to_white_space ( char *token, long line_length )
-{
-    char *orig_ptr = token;
-
-
-    while ( line_length > 0 && *token != ' ' && *token != '\t' )
-    {
-        line_length--;
-        token++;
-    }
-    
-    return ( token - orig_ptr );
-}
-
 
 /*
  * Return pointer to desired item in POP structure
@@ -919,6 +909,16 @@ get_opt_ptr ( POP *p, opt_val_type item )
         case kLOCKCHECK:           R__INT ( &p->check_lock_refresh );
         case kCLEARPASS:           R__MNM ( &p->AllowClearText );
         case kREVLOOKUP:           R__INT ( &no_rev_lookup );
+#ifdef    CYRUS_SASL
+        case kSASL_LOG_LOGIN:      R__PTR ( &p->sasl.sasl_log_login );
+        case kSASL_FORWARD_SEC:    R__BOO ( &p->sasl.sasl_forward_secrecy );
+        case kSASL_MIN_SSF:        R__INT ( &p->sasl.sasl_min_ssf );
+        case kSASL_MAX_SSF:        R__INT ( &p->sasl.sasl_max_ssf );
+        case kSASL_NO_ACTIVE:      R__BOO ( &p->sasl.sasl_no_active );
+        case kSASL_NO_ANONYMOUS:   R__BOO ( &p->sasl.sasl_no_anonymous );
+        case kSASL_NO_PLAINTEXT:   R__BOO ( &p->sasl.sasl_no_plaintext );
+        case kSASL_NO_DICTIONARY:  R__BOO ( &p->sasl.sasl_no_dictionary );
+#endif /* CYRUS_SASL */
         case kSTATS:               R__BOO ( &p->bStats );
         case kSERVER:              R__BOO ( &p->server_mode );
         case kTRACENAME:           R__PTR ( &p->trace_name );
@@ -952,6 +952,7 @@ get_opt_ptr ( POP *p, opt_val_type item )
         case kKEEP_TEMP_DROP:      R__BOO ( &p->bKeep_temp_drop );
         case kGRP_SERV_MODE:       R__PTR ( &p->grp_serv_mode );
         case kGRP_NO_SERV_MODE:    R__PTR ( &p->grp_no_serv_mode );
+        case kEXT_POSTAUTH_CMD:    R__PTR ( &p->ext_postauth_cmd );
         case kAUTHFILE:            R__PTR ( &p->authfile );
         case kNONAUTHFILE:         R__PTR ( &p->nonauthfile );
         case kSHY:                 R__BOO ( &p->bShy );
@@ -967,6 +968,7 @@ get_opt_ptr ( POP *p, opt_val_type item )
         case kLOG_LOGIN:           R__PTR ( &p->pLog_login );
         case kMAXBULLS:            R__INT ( &p->nMaxBulls );
         case kXTND_XMIT:           R__BOO ( &p->bXtnd_Xmit );
+        case kCONS_CACHE:          R__BOO ( &p->bCons_cache );
 #if 0 /* can't use this until we have a way to call pam_close_session with privs */
         case kPAM_SESSION:         R__MNM ( &p->call_pam_session );
 #endif
@@ -979,47 +981,56 @@ get_opt_ptr ( POP *p, opt_val_type item )
 
 
 /*
- * Return length of string type token
+ *  Get the next token
  */
 static long
-get_string_length ( char **token, long *line_length, char *buffer )
+get_next_token ( char **ptok, char **ntok )
 {
-    int len = 0;
+    int line_len, tok_len;
+    char term;
 
+    tok_len = 0;
+    skip_white_space (ntok);
+    line_len =  strlen(*ntok);
+    if (line_len == 0 ) return 0;
+    *ptok = *ntok;
 
-    if ( char_is_quote ( *token, buffer ) )
+    if ( **ntok == '=' || **ntok == COMMENT_CHAR )
     {
-        len          = length_in_quoted_str ( *token, *line_length );
-        *token       = *token + 1;
-        *line_length = *line_length - 1;
+        *ntok = *ptok + 1;
+        return 1;
     }
-    else
-        len = length_to_white_space ( *token, *line_length );
 
-    return len;
-}
-
-
-
-/*
- * Return length of current token.
- */
-static long
-get_token_length ( char *token, long line_length )
-{
-    char *orig_ptr = token;
-
-
-    if ( line_length > 0 && CharIsAlnum ( *token ) == FALSE )
-        return 1; /* non-alphanumerics are their own token */
-
-    while ( line_length > 0 && ( CharIsAlnum ( *token ) || *token == '-' ) )
+    if ( char_is_quote (*ntok) )
     {
-        token++;
-        line_length--;
+        term = **ntok;
+        *ntok = *ntok + 1;
+        *ptok = *ntok;
+        line_len--;
+        while ( line_len > 0 && **ntok != term )
+        {
+            if ( **ntok == QUOTE_CHAR )
+            {
+                *ntok = *ntok + 1;
+                line_len--;
+                tok_len++;
+            }
+            *ntok = *ntok+1;
+            line_len--;
+            tok_len++;
+        }
+    if ( **ntok == term) *ntok = *ntok + 1;
+    return (tok_len);
     }
-    
-    return ( token - orig_ptr );
+
+    while ( line_len > 0 && (**ntok != ' '  && 
+                             **ntok != '\t' && **ntok != '=') )
+    {
+        *ntok = *ntok+1;
+        line_len--;
+        tok_len++;
+    }
+    return ( tok_len );
 }
 
 
@@ -1027,32 +1038,19 @@ get_token_length ( char *token, long line_length )
  * Skips past white space.
  */
 static void
-skip_white_space ( char **token, long *line_length )
+skip_white_space ( char **token)
 {
-    while ( *line_length > 0 && ( **token == ' ' || **token == '\t' ) )
+    long len;
+
+    len = strlen (*token);
+    while ( len > 0 && ( **token == ' ' || **token == '\t' ) )
     {
-        *line_length   = *line_length   -1;
         *token         = *token         +1;
+        len--;
     }
 }
 
 
-/*
- * Skips to next token.
- */
-static void
-skip_token ( config_opt_type ttype, char **token, long *token_len, long *line_len )
-{
-    if ( ttype == CfgStr && ( **token == '\"' || 
-                              **token == '`'  || 
-                              **token != '\''   ) )
-        *token_len = *token_len + 1; /* account for close quote */
-
-    *token    += *token_len;
-    *line_len -= *token_len;
-    skip_white_space ( token, line_len );
-    *token_len = get_token_length ( *token, *line_len );
-}
 
 
 /*
@@ -1071,6 +1069,7 @@ show_result ( POP *p, config_table *opt, int iVal, const char *sVal, WHENCE )
             break;
 
         case CfgInt:
+        case CfgIntNeg:
             if ( p->debug )
                 pop_log ( p, POP_DEBUG, fn, ln, "Set %s to %d (%#0x)",
                           opt->name, iVal, iVal );
@@ -1126,6 +1125,21 @@ handle_value ( POP              *p,
     /*
      * Resolve the value
      */
+   if ( plen >= 2 && *pval == '-' && isdigit ( (int) *(pval +1) ) )
+    {
+        char *EndPtr = NULL;
+        iVal     = strtol ( pval, &EndPtr, 0 );
+        if ( iVal == LONG_MAX || iVal == LONG_MIN 
+                              || ( iVal == 0 && errno == EINVAL ) )
+        {
+            pop_log ( p, POP_PRIORITY, HERE,
+                      "Invalid value for %s: %.*s",
+                      opt_ent->name, plen, pval );
+            return kER_BAD_VAL;
+        }
+        val_type = CfgIntNeg;
+    }
+    else
     if ( isdigit ( (int) *pval ) )
     {
         char *EndPtr = NULL;
@@ -1231,6 +1245,18 @@ handle_value ( POP              *p,
             return kER_NOT_COMPILED_WITH;
 #endif /* ! ( defined ( QPOP_OPENSSL ) && defined ( SSL_OP_ALL ) ) */
 
+#ifndef    CYRUS_SASL
+        case     kSASL_LOG_LOGIN:
+        case     kSASL_MIN_SSF:
+        case     kSASL_MAX_SSF:
+        case     kSASL_NO_PLAINTEXT:
+        case     kSASL_NO_ACTIVE:
+        case     kSASL_NO_DICTIONARY:
+        case     kSASL_FORWARD_SEC:
+        case     kSASL_NO_ANONYMOUS:
+            return kER_NOT_COMPILED_WITH;
+#endif  /* CYRUS_SASL */
+
         default:
             break;
     } /* switch ( opt_ent->value ) */
@@ -1280,6 +1306,18 @@ handle_value ( POP              *p,
             show_result ( p, opt_ent, *pOpt.pInt, NULL, HERE );
             break;
 
+        case CfgIntNeg:
+            if ( val_type != CfgInt && val_type != CfgIntNeg )
+                return kER_EXP_INT_NEG;
+
+            pOpt = get_opt_ptr ( p, opt_ent->value );
+            if ( pOpt.pInt == NULL )
+                return kER_INTERNAL;
+
+            *pOpt.pInt  = iVal;
+            show_result ( p, opt_ent, *pOpt.pInt, NULL, HERE );
+            break;
+
         case CfgMnem:
             if ( val_type != CfgStr )
                 return kER_EXP_MNEM;
@@ -1380,9 +1418,10 @@ pop_result
 pop_config ( POP *p, char *config_file, config_call_type CallTime )
 {
     FILE           *config     = NULL;
-    size_t          line_no    = 1;
+    size_t          line_no    = 0;
     char            buffer [ MAXLINELEN ] = "";
     char           *ptok       = NULL;
+    char           *ntok       = NULL;
     long            line_len   = 0;
     long            tok_len    = 0;
     int             bReset     = FALSE;
@@ -1408,10 +1447,11 @@ pop_config ( POP *p, char *config_file, config_call_type CallTime )
         goto Exit;
     }
 
-    ptok = fgets ( buffer, MAXLINELEN -1, config );
-    while ( ptok != NULL )
+    rslt = POP_SUCCESS;
+    while ( (ntok = fgets ( buffer, MAXLINELEN -1, config ) ) != NULL )
     {
-        line_len = strlen ( ptok );
+        line_no++;
+        line_len = strlen ( ntok );
         
         /*
          * Make sure line was not too long for buffer.  Also
@@ -1419,31 +1459,26 @@ pop_config ( POP *p, char *config_file, config_call_type CallTime )
          */
         if ( buffer [ line_len -1 ] != '\n' )
         {
-            tilt ( p, kER_LINE_LEN, ptok, tok_len, line_no, config_file, opt, HERE );
-            goto Exit;
+            tilt ( p, kER_LINE_LEN, ntok, tok_len, line_no, config_file, opt, HERE );
+            rslt = POP_FAILURE;
+            continue;
         }
         buffer [ --line_len ] = '\0';
         
         /*
-         * Skip leading white space.
+         * Ignore empty lines and comment lines.
          */
-        skip_white_space ( &ptok, &line_len );
-        DEBUG_LOG3 ( p, "...read line %lu (%lu): %.256s", 
-                    line_no, line_len, ptok );
-        
-        /*
-         * Ignore empty and comment lines.
-         */
-        if ( line_len != 0 && *ptok != COMMENT_CHAR )
+        skip_white_space (&ntok);
+        if ( line_len != 0 && *ntok != COMMENT_CHAR)
         { /* process the line */
 
             /*
-             * Get token length.
+             * Get next token.
              */
-            tok_len = get_token_length ( ptok, line_len );
+            tok_len = get_next_token ( &ptok, &ntok );
 
             /*
-             * There should be a "set" (or even "reset" if
+             * There should be a "set" (or "reset" if
              * the option is boolean) before the name.
              */
             if ( equal_strings ( ptok, tok_len, "reset", 5 ) == TRUE )
@@ -1454,13 +1489,14 @@ pop_config ( POP *p, char *config_file, config_call_type CallTime )
             else
             {
                 tilt ( p, kER_EXP_SET, ptok, tok_len, line_no, config_file, opt, HERE );
-                goto Exit;
+                rslt = POP_FAILURE;
+                continue;
             }
             
             /*
              * Skip to next token.
              */
-            skip_token ( CfgBad, &ptok, &tok_len, &line_len );
+            tok_len = get_next_token (&ptok, &ntok);
 
             /*
              * See if the option is known.
@@ -1469,7 +1505,8 @@ pop_config ( POP *p, char *config_file, config_call_type CallTime )
             if ( opt == NULL )
             {
                 tilt ( p, kER_UNKN_OPT, ptok, tok_len, line_no, config_file, opt, HERE );
-                goto Exit;
+                rslt = POP_FAILURE;
+                continue;
             }
             
             /*
@@ -1478,7 +1515,8 @@ pop_config ( POP *p, char *config_file, config_call_type CallTime )
             if ( CallTime == CfgUser && opt->restriction == CfgResUser )
             {
                 tilt ( p, kER_RESTR, ptok, tok_len, line_no, config_file, opt, HERE );
-                goto Exit;
+                rslt = POP_FAILURE;
+                continue;
             }
 
             /*
@@ -1487,7 +1525,8 @@ pop_config ( POP *p, char *config_file, config_call_type CallTime )
             if ( CallTime == CfgConnected && opt->restriction == CfgResInit )
             {
                 tilt ( p, kER_CONNECTED, ptok, tok_len, line_no, config_file, opt, HERE );
-                goto Exit;
+                rslt = POP_FAILURE;
+                continue;
             }
 
             /*
@@ -1496,18 +1535,20 @@ pop_config ( POP *p, char *config_file, config_call_type CallTime )
             if ( bReset == TRUE && opt->type != CfgBool )
             {
                 tilt ( p, kER_NOT_BOOL, ptok, tok_len, line_no, config_file, opt, HERE );
-                goto Exit;
+                rslt = POP_FAILURE;
+                continue;
             }
 
             /*
              * See if there is an "=" followed by a value.
              */
-            skip_token ( CfgBad, &ptok, &tok_len, &line_len );
+            tok_len = get_next_token (&ptok, &ntok);
             if ( ( tok_len == 0 || *ptok != '=' )
                  && opt->type != CfgBool )
             {
                 tilt ( p, kER_EXP_EQL, ptok, tok_len, line_no, config_file, opt, HERE );
-                goto Exit;
+                rslt = POP_FAILURE;
+                continue;
             }
 
             if ( tok_len == 1 && *ptok == '=' )
@@ -1515,17 +1556,15 @@ pop_config ( POP *p, char *config_file, config_call_type CallTime )
                 /*
                  * Try and handle the value.
                  */
-                skip_token ( CfgBad, &ptok, &tok_len, &line_len );
-                if ( opt->type == CfgStr )
-                    tok_len = get_string_length ( &ptok, &line_len, buffer );
+                tok_len = get_next_token (&ptok, &ntok);
 
                 err = handle_value ( p, opt, ptok, tok_len, CallTime );
                 if ( err != kER_SUCCESS )
                 {
                     tilt ( p, err, ptok, tok_len, line_no, config_file, opt, HERE );
-                    goto Exit;
+                    rslt = POP_FAILURE;
+                    continue;
                 }
-                skip_token ( opt->type, &ptok, &tok_len, &line_len ); /* consume the token */
             }
             else
             if ( tok_len == 0 || *ptok == COMMENT_CHAR )
@@ -1537,35 +1576,31 @@ pop_config ( POP *p, char *config_file, config_call_type CallTime )
                 if ( err == POP_FAILURE )
                 {
                     tilt ( p, kER_BAD_VAL, ptok, tok_len, line_no, config_file, opt, HERE );
-                    goto Exit;
+                    rslt = POP_FAILURE;
+                    continue;
                 }
             } /* implicit value */
             else
             {
                 tilt ( p, kER_EXP_EQL, ptok, tok_len, line_no, config_file, opt, HERE );
-                goto Exit;
+                rslt = POP_FAILURE;
+                continue;
             }
             
             /*
              * Make sure there is no extra junk after the value
              */
+            tok_len = get_next_token (&ptok, &ntok);
             if ( tok_len != 0 && *ptok != COMMENT_CHAR )
             {
                 tilt ( p, kER_EXP_EOL, ptok, tok_len, line_no, config_file, opt, HERE );
-                goto Exit;
+                rslt = POP_FAILURE;
+                continue;
             }
 
         } /* process the line */
-
-        /*
-         * get the next line, if any
-         */
-        ptok = fgets ( buffer, MAXLINELEN -1, config );
-        line_no++;
-    } /* while ( ptok != NULL ) */
+    } /* while ((ntok = fgets ( buffer, MAXLINELEN -1, config )) != NULL ) */
     
-    rslt = POP_SUCCESS;
-
 Exit:
     /*
      * cleanup
@@ -1651,7 +1686,7 @@ check_config_files ( POP *p, struct passwd *pwp )
             rslt_usr = rslt_spl = POP_FAILURE; /* SETEUID failed */
             pop_log ( p, POP_PRIORITY, HERE,
                       "SETEUID(%i) for user %s failed",
-                      pwp->pw_uid, p->user );
+                      (int) pwp->pw_uid, p->user );
         } /* SETEUID failed */
         else { /* we are now the user */
             if ( bUser )
@@ -1661,7 +1696,7 @@ check_config_files ( POP *p, struct passwd *pwp )
 
             if ( SETEUID ( uid_save ) != 0 )
                 pop_log ( p, POP_PRIORITY, HERE,
-                          "SETEUID(%i) back failed", uid_save );
+                          "SETEUID(%i) back failed", (int) uid_save );
         } /* we are now the user */
     } /* bUser || bSpool */
 
@@ -1688,7 +1723,7 @@ set_option ( POP *p, char *opt_name, char *opt_value )
     pop_result      err        = POP_FAILURE;
 
 
-    DEBUG_LOG4 ( p, "set_option; opt_name: %lu '%s'; opt_value: %lu '%s'",
+    DEBUG_LOG4 ( p, "set_option; opt_name: %zu '%s'; opt_value: %zu '%s'",
                  strlen(opt_name), opt_name, strlen(opt_value), opt_value );
 
     /*
@@ -1709,3 +1744,4 @@ Exit:
 
     return rslt;
 }
+
