@@ -1,6 +1,6 @@
 
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
@@ -9,6 +9,9 @@
  * 07/12/09  [rcg]
  *         - Stop using sys_nerr and sys_errlist.
  *
+ * 06/28/08  [rcg]
+ *         - Patch from A'rpi to use cache even if new mail arrived. 
+ *
  * 03/09/01  [rcg]
  *         - Now including continuation of UIDL headers in UIDL hash.
  *         - Don't include random component of extra headers in UIDL
@@ -415,9 +418,13 @@ encode_uid_hash ( char *output, unsigned char *digest, BOOL bOld_style )
 
 /* 
  * Build the Msg_info_list from the temp_drop in case the previous session
- * failed to truncate. init_dropinfo and do_drop_copy differ in that
- * do_drop_copy copies all the messages from the source file to the 
+ * failed to truncate. init_dropinfo() and do_drop_copy() differ in that
+ * do_drop_copy() copies all the messages from the source file to the 
  * destination file while it generates the Msg_info_list.
+ *
+ * Note that the temp_drop is read from its current point.  This will be
+ * the start of the file, or, if a cache is used, the first new message
+ * not in the cache.
  */
 static int 
 init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
@@ -442,9 +449,11 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
     int             uw_hint    = 0;
     BOOL            bNewUIDs   = p->bUpdate_status_hdrs == TRUE &&
                                  p->bOld_style_uid      == FALSE;
+    long            f_pos      = 0;
 
     DEBUG_LOG1 ( p, "DROPINFO Checking file %s", fname );
-
+    
+    f_pos = ftell ( fdesc );
 
 #ifdef NULLKLUDGE
     {
@@ -468,7 +477,7 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
     if ( *buffer == MMDF_SEP_CHAR ) {
         p->mmdf_separator = (char *)strdup(buffer);
     } else if ( !isfromline(buffer) ) {
-        DEBUG_LOG5 ( p, "newline=%d; isOflow=%d; wasOflow=%d; buffer(%lu):  "
+        DEBUG_LOG5 ( p, "newline=%d; isOflow=%d; wasOflow=%d; buffer(%zu):  "
                         "%.256s", 
                      newline, isOflow, wasOflow, strlen(buffer), buffer );
         return pop_msg ( p, POP_FAILURE, HERE,
@@ -479,18 +488,17 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
     }
 
     newline = 1;
-    rewind ( fdesc );
+    fseek ( fdesc, f_pos, SEEK_SET );
 
     bInHeader         = FALSE;
     bInContHeader     = FALSE;
-    msg_num           = 0;
-    visible_msg_num   = 0;
+    msg_num           = p->msg_count;
+    visible_msg_num   = p->visible_msg_count;
     uidl_found        = 0;
     expecting_trailer = 0;
     content_length    = 0;
     content_nchar     = 0;
     cont_len          = 0;
-    p->msg_count      = ALLOC_MSGS;
 
 
     if ( DEBUGGING && time_locked == 0 )
@@ -507,9 +515,11 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
     }
 #endif
 
-    DEBUG_LOG1 ( p, "Building the Message Information list from %s", fname );
+    DEBUG_LOG2 ( p, "Building the Message Information list from %s"
+                    " starting at offset %ld", fname, f_pos );
 
-    for ( mp = p->mlp - 1; mfgets ( buffer, MAXMSGLINELEN, fdesc ); ) {
+    mp = p->mlp + msg_num - 1;
+    while ( mfgets ( buffer, MAXMSGLINELEN, fdesc ) ) {
         nchar = strlen(buffer);
         
         /*
@@ -635,22 +645,22 @@ init_dropinfo ( POP *p, char *fname, FILE *fdesc, time_t time_locked )
                 if ( !uidl_found ) {
                     char    *cp;
 
-                    if ( p->bOld_style_uid == FALSE )
-                        MD5Update ( &mdContext, (unsigned char *)frombuf, 
-                                    strlen(frombuf) );
-    
+                if ( p->bOld_style_uid == FALSE )
+                    MD5Update ( &mdContext, (unsigned char *)frombuf, 
+                                strlen(frombuf) );
+
                     MD5Final ( digest, &mdContext );
                     cp = mp->uidl_str;
                     cp = encode_uid_hash ( cp, digest, p->bOld_style_uid );
                     *cp++ = '\n';
                     *cp   = '\0';
                     
-                    DEBUG_LOG2 ( p, "UID not found; generated UID(%lu): %s",
+                    DEBUG_LOG2 ( p, "UID not found; generated UID(%zu): %s",
                                     strlen(mp->uidl_str), mp->uidl_str );
-    
+
                     mp->length   += strlen("X-UIDL: ") + strlen(mp->uidl_str) +1;
                     p->drop_size += strlen("X-UIDL: ") + strlen(mp->uidl_str) +1;
-    
+
                     /* New UIDs do not dirty the mailspool unless
                      * bUpdate_status_hdrs is set.  They are just recalculated
                      * each time Qpopper is run.
@@ -836,7 +846,7 @@ do_drop_copy ( POP *p, int mfd, time_t time_locked )
         if ( !p->mmdf_separator )
             p->mmdf_separator = (char *)strdup(buffer);
     } else if ( !isfromline ( buffer ) ) {
-        DEBUG_LOG5 ( p, "newline=%d; isOflow=%d; wasOflow=%d; buffer(%lu):  %.256s", 
+        DEBUG_LOG5 ( p, "newline=%d; isOflow=%d; wasOflow=%d; buffer(%zu):  %.256s", 
                      newline, isOflow, wasOflow, strlen(buffer), buffer );
         return pop_msg ( p, POP_FAILURE, HERE,
                             "[SYS/PERM] Unable to process From lines "
@@ -1024,7 +1034,7 @@ do_drop_copy ( POP *p, int mfd, time_t time_locked )
                     cp = encode_uid_hash ( cp, digest, p->bOld_style_uid );
                     *cp++ = '\n';
                     *cp   = '\0';
-                    DEBUG_LOG3 ( p, "UID not found; generated UID(%lu): %s [%d]",
+                    DEBUG_LOG3 ( p, "UID not found; generated UID(%zu): %s [%d]",
                                     strlen(mp->uidl_str), mp->uidl_str, __LINE__ );
 
                     mp->length   += strlen("X-UIDL: ") + strlen(mp->uidl_str) +1;
@@ -1292,7 +1302,7 @@ struct passwd   * pwp;
         if ( p->bull_db == NULL ) {
             pop_log (p, POP_PRIORITY, HERE, "gdbm_open failed: %s (%d)"
                                             "; bull_db: %s",
-                        STRERROR( errno ), errno, bull_db );
+                        sys_errlist[errno], errno, bull_db );
             if ( p->bulldb_nonfatal == FALSE )
                 return ( pop_msg ( p, POP_FAILURE, HERE, 
                                    "[SYS/TEMP] Unable to open Bulletin "
@@ -1722,8 +1732,11 @@ struct passwd   * pwp;
              * If we have a usable cache file, that saves us from having to
              * process the spool.
              */
-            rslt = cache_read ( p, fileno(p->drop), &p->mlp );
+            rslt = cache_read ( p, p->drop, &p->mlp );
             if ( rslt == POP_FAILURE ) {
+                DEBUG_LOG1 ( p, "cache_read returned %i; calling "
+                                "init_dropinfo to process temp drop",
+                             rslt );
                 rslt = init_dropinfo ( p, p->drop_name, p->drop, time_locked );
                 if ( rslt != POP_SUCCESS )
                     goto bailout;
@@ -1743,7 +1756,7 @@ struct passwd   * pwp;
          */
         p->server_mode = 0;
         DEBUG_LOG3 ( p, "Unable to open maildrop %s: %s (%d)", 
-                     p->drop_name, STRERROR(errno), errno );
+                     p->drop_name, STRERROR ( errno ), errno );
         cache_unlink ( p );
     }
 
