@@ -8,12 +8,18 @@
 
 
 /*
- * Copyright (c) 2008 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
  * Revisions:
  *
+ *     01/09/09 [RCG]
+ *              - Patch from Jason White to fix Kerberos error.
+ *
+ *     08/08/06 [RCG]
+ *              - IPv6 patch by Hajimu UMEMOTO
+ *
  *     04/11/05 [RCG]
  *              - Add 'x' command-line option to disable XTND XMIT.
  *
@@ -188,13 +194,14 @@
 #  endif /* KRB5 */
 #endif /* KERBEROS */
 
+#ifndef INET6
 /* Some systems define (in netdb.h) h_errno as get_h_errno() */
-#ifndef h_errno
+#  ifndef   h_errno
   extern int      h_errno; /* Error external for gethostxxxx library */
-#endif
+#  endif /* not h_errno */
 
 /* Method to get friendly string from h_errno */
-#ifndef HAVE_HSTRERROR
+#  ifndef HAVE_HSTRERROR
   const char * _HERRORS[] = 
       {"(No problem)",                                          /*  0  NETDB_SUCCESS  */
        "Authoritive answer: Host not found",                    /*  1  HOST_NOT_FOUND */
@@ -203,7 +210,8 @@
        "Valid name, no data record of requested type",          /*  4  NO_DATA        */
        "(Unknown)"};
 #  define hstrerror(x) ( (x >= 0 && x <= NO_DATA) ? _HERRORS[x] : "(unknown)" )
-#endif /* HAVE_HSTRERROR */
+#  endif /* HAVE_HSTRERROR */
+#endif /* not INET6 */
 
 extern int      errno;
        int      no_rev_lookup;      /*  Avoid reverse lookup? */
@@ -238,10 +246,51 @@ strdup(str)
  * in unistd.h */
 extern int gethostname ( );
 
+char *
+sock_ntop ( p, salen )
+    struct sockaddr *p;
+    int salen;
+{
+#ifdef INET6
+    static char hbuf[NI_MAXHOST];
+    int niflags;
+
+    niflags = NI_NUMERICHOST;
+#  ifdef    NI_WITHSCOPEID
+    if ( ((struct sockaddr *)p)->sa_family == AF_INET6 )
+        niflags |= NI_WITHSCOPEID;
+#  endif /* NI_WITHSCOPEID */
+    if ( getnameinfo ( p, salen, hbuf, sizeof(hbuf), NULL, 0, niflags ) )
+        return "(invalid)";
+    return hbuf;
+#else /* not INET6 */
+    return inet_ntoa ( ((struct sockaddr_in *)p)->sin_addr );
+#endif /* INET6 */
+    (void) salen; /* avoid 'unused parameter' warnings */
+}
+
+
+int sock_port ( struct sockaddr *p, int salen )
+{
+#ifdef INET6
+    static char pbuf[NI_MAXSERV];
+
+    if ( getnameinfo ( p, salen, NULL, 0, pbuf, sizeof(pbuf),
+                       NI_NUMERICSERV ) == 0 )
+        return atoi ( pbuf );
+    else
+        return 0;
+#else /* not INET6 */
+    return ntohs ( ((struct sockaddr_in *)p)->sin_port );
+#endif /* INET6 */
+    (void) salen; /* avoid 'unused parameter' warnings */
+}
+
 int
-kerb_authenticate ( p, addr )
+kerb_authenticate ( p, addr, salen )
      POP                *p;
-     struct sockaddr_in *addr;
+     struct sockaddr *addr;
+     int salen;
 {
 
 #ifdef KERBEROS
@@ -251,8 +300,9 @@ kerb_authenticate ( p, addr )
     char           instance[INST_SZ];  
     char           version[9];
     int            auth;
-  
-    if ( p->bKerberos ) {
+
+
+    if ( p->bKerberos && addr->sa_family == AF_INET ) {        /*XXX*/
 
 #    ifdef  K_GETSOCKINST
         k_getsockinst ( 0, instance );
@@ -261,7 +311,8 @@ kerb_authenticate ( p, addr )
 #    endif /* K_GETSOCKINST */
 
         auth = krb_recvauth ( 0L, 0, &ticket, p->kerberos_service, instance,
-                              addr, (struct sockaddr_in *) NULL,
+                              (struct sockaddr_in *) addr,
+                              (struct sockaddr_in *) NULL,
                               &kdata, "", schedule, version );
         
         if ( auth != KSUCCESS ) {
@@ -276,7 +327,7 @@ kerb_authenticate ( p, addr )
         } /* auth != KSUCCESS */
 
         DEBUG_LOG4 ( p, "%s.%s@%s (%s): ok", kdata.pname, 
-                     kdata.pinst, kdata.prealm, inet_ntoa(addr->sin_addr) );
+                     kdata.pinst, kdata.prealm, sock_ntop ( addr, salen ) );
 
         strncpy ( p->user, kdata.pname, sizeof(p->user) );
         p->AuthType = kerberos;
@@ -296,7 +347,7 @@ kerb_authenticate ( p, addr )
     krb5_int32         auth_sys      = 0;
     int                sock          = 0;
 
-    if ( p->bKerberos ) {
+    if ( p->bKerberos && addr->sa_family == AF_INET ) {        /*XXX*/
         retval = krb5_init_context ( &pop_context );
         if ( retval ) {
             pop_msg ( p, POP_FAILURE, HERE,
@@ -343,7 +394,7 @@ kerb_authenticate ( p, addr )
                                0,              /* V4 options */
                                p->kerberos_service,    /* V4 service */
                                v4_instance,    /* V4 instance */
-                               addr,           /* Remote address */
+                               (struct sockaddr_in *)addr, /* Remote address */
                                NULL,           /* Local address (unused) */
                                "",             /* Use default srvtab */
                                &ticket,        /* V5 ticket for client name */
@@ -381,7 +432,7 @@ kerb_authenticate ( p, addr )
                           error_message(retval) );
                 pop_log ( p, POP_DEBUG, HERE, 
                           "unable to convert V4 principal (%s)", 
-                          inet_ntoa(addr->sin_addr) );
+                          sock_ntop ( addr, salen ) );
                 EXIT ( -1 );
             } /* krb5_425_conv_principal failed */
         } /* auth_sys == KRB5_RECVAUTH_V4 */
@@ -400,7 +451,7 @@ kerb_authenticate ( p, addr )
                           error_message(retval) );
                 pop_log ( p, POP_DEBUG, HERE, 
                           "unable to copy principal (%s)",
-                          inet_ntoa(addr->sin_addr) );
+                          sock_ntop ( addr, salen ) );
                 EXIT ( -1 );
             } /* krb5_copy_principal failed */
 
@@ -411,7 +462,7 @@ kerb_authenticate ( p, addr )
             pop_msg ( p, POP_FAILURE, HERE, "unknown authentication type--%d",
                       auth_sys);
             pop_log ( p, POP_DEBUG, HERE, "unknown authentication type (%s)",
-                      inet_ntoa(addr->sin_addr) );
+                      sock_ntop ( addr, salen ) );
             EXIT ( -1 );
         }
 
@@ -420,12 +471,12 @@ kerb_authenticate ( p, addr )
             pop_msg ( p, POP_FAILURE, HERE, "name not parsable--%s",
                       error_message(retval) );
             pop_log ( p, POP_DEBUG, HERE, "name not parsable (%s)",
-                      inet_ntoa(addr->sin_addr) );
+                      sock_ntop ( addr, salen ) );
             EXIT ( -1 );
         } /* krb5_unparse_name failed */
 
         DEBUG_LOG2 ( p, "%s (%s): ok", 
-                     client_name, inet_ntoa(addr->sin_addr) );
+                     client_name, sock_ntop ( addr, salen ) );
 
 #    ifdef NO_CROSSREALM
         retval = krb5_aname_to_localname ( pop_context, ext_client, 
@@ -448,6 +499,7 @@ kerb_authenticate ( p, addr )
 #endif /* KERBEROS */
 
     return ( POP_SUCCESS );
+    (void) salen; /* avoid 'unused parameter' warnings */
 }
 
 /* 
@@ -462,8 +514,12 @@ int             argcount;
 char    **      argmessage;
 {
 
+#ifdef    INET6
+    struct sockaddr_storage cs;                 /*  Communication parameters */
+#else /* not INET6 */
     struct sockaddr_in      cs;                 /*  Communication parameters */
     struct hostent      *   ch;                 /*  Client host information */
+#endif /* INET6 */
     int                     errflag = 0;
     int                     c;
     int                     len;
@@ -472,7 +528,15 @@ char    **      argmessage;
     int                     sp = 0;             /*  Socket pointer */
     int                     pw_handling = 0;    /*  How clear text passwords handled */
     int                     tls_support = 0;    /*  Desired TLS/SSL support */
+#ifdef INET6
+    struct addrinfo         hints;
+    struct addrinfo     *   res;
+    struct addrinfo     *   r;
+    int                     error;
+    char                    hbuf[NI_MAXHOST];
+#else /* INET6 */
     struct hostent      *   hp = NULL;
+#endif /* INET6 */
 
 
 #if DISABLE_CANNONICAL_LOOKUP
@@ -508,7 +572,7 @@ char    **      argmessage;
 
 #ifdef     KERBEROS
     p->kerberos_service    = KERBEROS_SERVICE;
-    DEBUG_LOG1 ( p, "Set kerberos_service to %s", p-.kerberos_service );
+    DEBUG_LOG1 ( p, "Set kerberos_service to %s", p->.kerberos_service );
 #endif  /* KERBEROS */
 
 #ifdef    SERVER_MODE
@@ -1079,6 +1143,35 @@ char    **      argmessage;
     p->client = p->ipaddr;
 #else
     (void) gethostname ( p->myhost, MAXHOSTNAMELEN );
+#ifdef INET6
+    memset ( &hints, 0, sizeof ( hints ) );
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    hints.ai_flags = AI_PASSIVE | AI_CANONNAME;
+    res = NULL;
+    error = getaddrinfo ( p->myhost, NULL, &hints, &res );
+    if ( !error && res->ai_canonname != NULL ) {
+        if ( index ( res->ai_canonname, '.' ) == NULL ) {         /* FQN not returned */
+            /*
+             * SVR4 resolver is stupid and returns h_name as whatever
+             * you gave gethostbyname.  Thus do a reverse lookup
+             * on the first address and hope for the best.
+             */
+            error = getnameinfo ( res->ai_addr, res->ai_addrlen,
+                                  hbuf, sizeof(hbuf), NULL, 0, NI_NAMEREQD );
+            if ( !error ) {
+                (void) strncpy ( p->myhost, hbuf, MAXHOSTNAMELEN );
+                p->myhost [ MAXHOSTNAMELEN ] = '\0';
+            }
+        } /* index ( hp->h_name, '.' ) == NULL */
+        else {
+            (void) strncpy ( p->myhost, res->ai_canonname, MAXHOSTNAMELEN );
+            p->myhost [ MAXHOSTNAMELEN ] = '\0';
+        }
+    }
+    if ( res )
+        freeaddrinfo ( res );
+#else /* INET6 */
     hp = gethostbyname ( p->myhost );
     if ( hp != NULL ) {
         if ( index ( hp->h_name, '.' ) == NULL ) {         /* FQN not returned */
@@ -1107,28 +1200,49 @@ char    **      argmessage;
             p->myhost [ MAXHOSTNAMELEN ] = '\0';
         }
     }
+#endif /* INET6 */
 
     /*  
      * Get the address and socket of the client to whom I am speaking 
      */
     len = sizeof(cs);
-    if ( getpeername ( sp, (struct sockaddr *) &cs, &len ) < 0 ) {
+    if ( getpeername ( sp, (struct sockaddr *)&cs, &len ) < 0 ) {
         pop_log ( p, POP_PRIORITY, HERE,
                   "Unable to obtain socket and address of client: %s (%d)",
                   STRERROR(errno), errno );
         EXIT ( 1 );
     }
 
+#ifdef    INET6
+    if ( ((struct sockaddr *)&cs)->sa_family == AF_INET6 &&
+         IN6_IS_ADDR_V4MAPPED( &((struct sockaddr_in6 *)&cs)->sin6_addr ) ) {
+        struct sockaddr_in sin;
+        struct sockaddr_in6 *sin6;
+        const int off = sizeof(struct in6_addr) - sizeof(struct in_addr);
+
+        sin6 = (struct sockaddr_in6 *)&cs;
+        sin.sin_family = AF_INET;
+        len = sizeof(struct sockaddr_in);
+#ifdef SIN6_LEN
+        sin.sin_len = len;
+#endif
+        memcpy ( &sin.sin_addr, &sin6->sin6_addr.s6_addr[off],
+                 sizeof(sin.sin_addr) );
+        sin.sin_port = sin6->sin6_port;
+        memcpy ( &cs, &sin, sizeof(sin) );
+    }
+#endif /* INET6 */
+
     /*
      * Save the dotted decimal form of the client's IP address 
      * in the POP parameter block 
      */
-    p->ipaddr = (char *) strdup ( inet_ntoa ( cs.sin_addr ) );
+    p->ipaddr = (char *) strdup ( sock_ntop ( (struct sockaddr *)&cs, len ) );
 
     /*  
      * Save the client's port 
      */
-    p->ipport = ntohs ( cs.sin_port );
+    p->ipport = sock_port ( (struct sockaddr *)&cs, len );
 
     if ( no_rev_lookup )
         p->client = p->ipaddr;
@@ -1136,6 +1250,16 @@ char    **      argmessage;
         /*  
          * Get the canonical name of the host to whom I am speaking 
          */
+#ifdef    INET6
+        error = getnameinfo ( (struct sockaddr *)&cs, len,
+                              hbuf, sizeof(hbuf), NULL, 0, NI_NAMEREQD );
+        if ( error ) {
+            pop_log ( p, POP_DEBUG, HERE,
+                      "(v%s) Unable to get canonical name of client %s: %s (%d)",
+                      VERSION, p->ipaddr, gai_strerror(error), error );
+            p->client = p->ipaddr;
+        }
+#else /* INET6 */
         ch = gethostbyaddr ( (char *) &cs.sin_addr, sizeof(cs.sin_addr), AF_INET );
         if ( ch == NULL ) {
             pop_log ( p, POP_DEBUG, HERE,
@@ -1143,40 +1267,98 @@ char    **      argmessage;
                       VERSION, p->ipaddr, hstrerror(h_errno), h_errno );
             p->client = p->ipaddr;
         }
+#endif /* INET6 */
         /*  
          * Save the cannonical name of the client host in 
          * the POP parameter block 
          */
         else {
 
-#ifndef BIND43
-            p->client = (char *) strdup ( ch->h_name );
-#else
+#ifdef    INET6
+#  ifndef BIND43
+            p->client = (char *) strdup ( hbuf );
+#  else /* BIND43 */
 
-# ifndef SCOR5
+            char h_name[MAXHOSTNAMELEN + 1];
+
+            strlcpy ( h_name, hbuf, sizeof(h_name) );
+
+            /*  See if the name obtained for the client's IP 
+                address returns an address */
+            memset ( &hints, 0, sizeof ( hints ) );
+            hints.ai_family = ((struct sockaddr *)&cs)->sa_family;
+            hints.ai_socktype = SOCK_STREAM;
+            hints.ai_flags = AI_PASSIVE | AI_CANONNAME;
+            res = NULL;
+            error =  getaddrinfo ( h_name, NULL, &hints, &res );
+            if ( error || res->ai_canonname == NULL ) {
+                pop_log (p, POP_PRIORITY, HERE,
+                         "Client at \"%s\" resolves to an unknown host name \"%s\"",
+                         p->ipaddr, h_name );
+                p->client = p->ipaddr;
+            }
+            else {
+                /*  Save the host name (the previous value was 
+                    destroyed by gethostbyname) */
+                p->client = (char *)strdup(res->ai_canonname);
+
+                memset ( &r, 0, sizeof ( r ) );
+                /*  Look for the client's IP address in the list returned 
+                    for its name */
+                for ( r = res; r; r = r->ai_next ) {
+                    int niflags = NI_NUMERICHOST;
+#  ifdef    NI_WITHSCOPEID
+                    if (r->ai_addr->sa_family == AF_INET6)
+                        niflags |= NI_WITHSCOPEID;
+#  endif /* NI_WITHSCOPEID */
+                    /* We don't use sock_ntop() here due to avoid matching
+                     * error wrongly */
+                    error = getnameinfo ( r->ai_addr, r->ai_addrlen,
+                                          hbuf, sizeof(hbuf), NULL, 0,
+                                          niflags );
+                    if ( !error && strcmp ( hbuf, p->ipaddr ) == 0 ) break;
+                }
+            }
+            if ( res )
+                freeaddrinfo ( res );
+            if ( !r ) {
+                pop_log ( p, POP_PRIORITY, HERE,
+                          "Client address \"%s\" not listed for its "
+                          "host name \"%s\"",
+                          p->ipaddr, h_name );
+                p->client = p->ipaddr;
+            }
+
+#  endif /* BIND43 */
+#else /* not INET6 */
+#  ifndef BIND43
+            p->client = (char *) strdup ( ch->h_name );
+#  else /* not BIND43 */
+
+#   ifndef   SCOR5
 #       include <arpa/nameser.h>
 #       include <resolv.h>
-# endif
+#   endif /* SCOR5 */
 
         /*  Distrust distant nameservers */
 
-#if !(defined(BSD) && (BSD >= 199103)) && !defined(OSF1) && !defined(HPUX10)
-# if (!defined(__RES)) || (__RES < 19940415)
-#  ifdef SCOR5
+#  if !(defined(BSD) && (BSD >= 199103)) && !defined(OSF1) && !defined(HPUX10)
+#   if (!defined(__RES)) || (__RES < 19940415)
+#    ifdef    SCOR5
         extern struct __res_state       _res;
-#  else
+#    else
         extern struct state         _res;
-#  endif
-# endif
-#endif
+#    endif /* SCOR5 */
+#   endif /* __RES < 19940415 */
+#  endif /* BSD >= 199103 and not OSF1 and not HPUX10 */
         struct hostent      *   ch_again;
         char            *   *   addrp;
         char                    h_name[MAXHOSTNAMELEN + 1];
 
         /*  We already have a fully-qualified name */
-#ifdef RES_DEFNAMES
+#  ifdef RES_DEFNAMES
         _res.options &= ~RES_DEFNAMES;
-#endif
+#  endif
 
         strncpy(h_name, ch->h_name, sizeof(h_name));
 
@@ -1206,15 +1388,16 @@ char    **      argmessage;
                 }
         }
 
-#ifdef RES_DEFNAMES
+#    ifdef RES_DEFNAMES
         /* 
          *  Must restore nameserver options since code in crypt uses
          *  gethostbyname call without fully qualified domain name!
          */
         _res.options |= RES_DEFNAMES;
-#endif
+#    endif
 
-#endif /* BIND43 */
+#  endif /* BIND43 */
+#endif /* INET6 */
                 }
 
     } /* no_rev_lookup */
@@ -1238,6 +1421,6 @@ char    **      argmessage;
 
     pop_log ( p, POP_DEBUG, HERE, "(v%s) Servicing request from \"%s\" at %s",
               VERSION, p->client, p->ipaddr );
-    return ( kerb_authenticate ( p, &cs ) );
+    return ( kerb_authenticate ( p, (struct sockaddr *)&cs, len ) );
 }
 
