@@ -6,7 +6,7 @@
 
 
 /*
- * Copyright (c) 2005 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
@@ -65,7 +65,7 @@ POP     * p;
     register int            i;
     register int            msg_num;
     int                     show_mangled_length = 0;
-    unsigned long           mangled_length      = 0;
+    long                    mangled_length      = 0;
     char                    buffer [ MAXMSGLINELEN ];
     MimeParsePtr            mimeParse;
     ManglerStateType        mangleState;
