@@ -6,7 +6,7 @@
 
 
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
@@ -98,7 +98,7 @@ pop_logv ( POP         *p,         /* the source of all things */
 {
     char            *date_time = NULL;
     struct timeval   tval;
-    size_t           iLeft     = 0;
+    int              iLeft     = 0;
     int              iChunk    = 0;
     int              iDateLen  = 0;
     long             lMsec     = 0;
@@ -119,7 +119,7 @@ pop_logv ( POP         *p,         /* the source of all things */
         char whence [ 512 ];
         int  len;
         
-        iLeft -= ( iChunk >= 0 ? iChunk : strlen(msgbuf) );
+        iLeft -= ( iChunk >= 0 ? iChunk : (int) strlen(msgbuf) );
         len    = Qsprintf ( whence, " [%s:%lu]", fn, (long unsigned) ln );
         strncat ( msgbuf, whence, iLeft );
         msgbuf [ sizeof(msgbuf) -1 ] = '\0'; /* just to make sure */
