@@ -1,10 +1,14 @@
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
  * Revisions:
  *
+ *     04/23/08  [rcg]
+ *             - Move macro search and replace to a new function.
+ *               Fixes compilation errors on some compilers.
+ *
  *     06/01/01  [rcg]
  *             - Fixed empty password treated as empty command
  *               (patch submitted by Michael Smith and others).
@@ -35,16 +39,48 @@
 
 #include "popper.h"
 
+
+/*
+ * pop_macro_replace:
+ * 
+ * Search macro array for a macro whose name is parameter 'pinx'.
+ * If found, replace parameter 'pinx' with macro value.
+ * Returns macro index if found, 0 otherwise.
+ */
+int pop_macro_replace ( POP *p, int pinx )
+{
+     /*
+      * Search in the Mdef Array for the presence of this arg.
+      */
+     int j;
+     extern MDEFRecordType mdrArr[MAX_MDEF_INDEX];
+     extern int last_index;
+    
+     for( j = 0; j <= last_index; j++ )
+     {
+         if ( strcasecmp(mdrArr[j].mdef_macro, p->pop_parm[pinx]) == 0 )
+         {
+             DEBUG_LOG4 ( p, "Parameter %i matched macro %i: %.50s (%.50s)",
+                          pinx, j, mdrArr[j].mdef_macro,
+                          mdrArr[j].mdef_value );
+             p->pop_parm[pinx] = mdrArr[j].mdef_value;
+             p->pop_parmlen[pinx] = strlen(p->pop_parm[pinx]);
+             return pinx;
+         }
+     }
+    return 0;
+}
+
+
+
 /* 
  *  parse:  Parse a raw input line from a POP client 
- *  into null-delimited tokens
+ *  into null-delimited tokens.
+ *
+ *  'buf' is a pointer to a message containing the line from the client
  */
 
-int
-pop_parse ( p, buf )
-POP         *   p;
-char        *   buf;        /*  Pointer to a message containing 
-                                the line from the client */
+int pop_parse ( POP *p, char *buf )
 {
     char            *   mp;
     register int        i;
@@ -52,7 +88,8 @@ char        *   buf;        /*  Pointer to a message containing
     /*
      * Loop through the POP command array 
      */
-    for (mp = buf, i = 0; ; i++) {
+    for (mp = buf, i = 0; ; i++)
+    {
     
         /*
          * Skip leading spaces and tabs in the message 
@@ -69,7 +106,8 @@ char        *   buf;        /*  Pointer to a message containing
         /*
          * Have we already obtained the maximum allowable parameters? 
          */
-        if ( i >= MAXPARMCOUNT ) {
+        if ( i >= MAXPARMCOUNT )
+        {
             pop_msg ( p, POP_FAILURE, HERE, "Too many arguments supplied." );
             return ( -1 );
         }
@@ -91,7 +129,8 @@ char        *   buf;        /*  Pointer to a message containing
         if ( *mp != 0 )
             *mp++ = 0;
 
-        if (i == 0) {
+        if (i == 0)
+        {
             /*
              * Convert the first token (POP command) to lower case
              */
@@ -103,36 +142,36 @@ char        *   buf;        /*  Pointer to a message containing
              * strcmp() on every call to pop_parse();  This parsing keeps
              * leading and trailing speces behind for the password command.
              */
-            if ( p->pop_command[0] == 'p' && strcmp(p->pop_command, "pass") == 0 ) {
-                if ( *mp != 0 ) {
+            if ( p->pop_command[0] == 'p' 
+                 && strcmp(p->pop_command, "pass") == 0 ) 
+            {
+                if ( *mp != 0 )
+                {
                     p->pop_parm[1] = mp;
-                    if ( strlen(mp) > 0 ) {
-                        mp = mp + strlen(mp) - 1;
+                    p->pop_parmlen[1] = strlen(mp);
+                    if ( p->pop_parmlen[1] > 0 )
+                    {
+                        mp = mp + p->pop_parmlen[1] - 1;
                         while ( *mp == 0xa || *mp == 0xd )
+                        {
                             *mp-- = 0;
+                            p->pop_parmlen[1]--;
+                        }
                     }
                     return ( 1 );
-                } else
+                }
+                else
                     return ( 0 );
             }
         } /* first parameter (command name) */
-        else {
+        else 
+        {
+            p->pop_parmlen[i] = strlen(p->pop_parm[i]);
+
             /*
              * Search in the Mdef Array for the presence of this arg.
              */
-            int j;
-            extern MDEFRecordType mdrArr[MAX_MDEF_INDEX];
-            extern int last_index;
-
-            for( j = 0; j <= last_index; j++ ) {
-                if ( strcasecmp(mdrArr[j].mdef_macro, p->pop_parm[i]) == 0 ) {
-                    DEBUG_LOG4 ( p, "Parameter %i matched macro %i: %.50s (%.50s)",
-                                 i, j, mdrArr[j].mdef_macro,
-                                 mdrArr[j].mdef_value );
-                    p->pop_parm[i] = mdrArr[j].mdef_value;
-                    break;
-                }
-            }
+            pop_macro_replace ( p, i );
         }
     }
 
