@@ -6,7 +6,7 @@
 
 
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
@@ -17,6 +17,14 @@
  *                dereferencing parameter 'msg' in function
  *                PAM_qpopper_conv.
  *
+ *     04/25/05  [dts]
+ *              - Integrate do_ext_postauth_cmd function and call
+ *                to it. This is useful for smtp-after-pop and
+ *                could be used for other things. Program specified
+ *                in config file is called with IP address and user
+ *                name. External program can do whatever it wants
+ *                with this information.
+ *
  *     04/22/05  [rcg]
  *              - Use generic auth error msg and sleep in more cases
  *
@@ -47,7 +55,10 @@
  *
  *     01/16/03  [rcg]
  *             - Renamed PASSWD macro to QPASSWD to avoid redefining
- *               PASSWD in shadow.h.
+ *               QPASSWD in shadow.h.
+ *
+ *     01/11/03  [pad]
+ *             - Added Cygwin section
  *
  *     06/01/01  [rcg]
  *             - Added patch by Clifton Royston to change error message
@@ -141,6 +152,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -1058,6 +1070,68 @@ Exit:
 #    endif /* AIX */
 
 
+/*----------------------------------------------- CYGWIN  */
+#    ifdef __CYGWIN__
+
+#      define DECLARED_AUTH_USER
+
+/*
+ * CYGWIN specific special authentication
+ *
+ * Use crypt on Win95, 98 and Me, and Windows on Nt, 2000 and XP
+ */
+
+/* Windows definitions 
+   Including windows.h generates many conflicts */
+typedef void *HANDLE;                     /* winnt.h */
+#define WINAPI __stdcall                  /* windef.h */
+typedef unsigned long DWORD;              /* windef.h */
+typedef int WINBOOL;                      /* windef.h */
+#define INVALID_HANDLE_VALUE (HANDLE)(-1) /* winbase.h */
+DWORD WINAPI GetVersion(void);            /* winbase.h */
+WINBOOL WINAPI CloseHandle(HANDLE);       /* winbase.h */
+# define WIN95_98_ME 0x80000000 
+
+/* From sys/cygwin.h */ 
+extern HANDLE cygwin_logon_user (const struct passwd *, const char *);
+extern void cygwin_set_impersonation_token (const HANDLE);
+
+static int
+auth_user ( POP *p, struct passwd *pw )
+{
+    if (GetVersion() & WIN95_98_ME) {
+        /*  
+         * We don't accept connections from users with null passwords 
+         *
+         *  Compare the supplied password with the password file entry 
+         */
+        if ( (  pw->pw_passwd == NULL ) || 
+             ( *pw->pw_passwd == '\0' ) ||
+             strcmp ( crypt ( p->pop_parm[1], pw->pw_passwd ), pw->pw_passwd )
+            ) {
+            sleep ( SLEEP_SECONDS );
+            return ( pop_msg ( p, POP_FAILURE, HERE, ERRMSG_PW, p->user ) );
+        }
+    }
+    else { /* NT_2000_XP */
+        HANDLE token;
+        token = cygwin_logon_user (pw, p->pop_parm[1]);
+        if (token == INVALID_HANDLE_VALUE) {
+            sleep ( SLEEP_SECONDS );
+            return ( pop_msg ( p, POP_FAILURE, HERE, ERRMSG_PW, p->user ) );
+        }
+#ifdef BINMAIL_IS_SETGID 
+                CloseHandle (token); /* Will need an internal token */
+#else
+        cygwin_set_impersonation_token (token);
+#endif
+    }
+    return ( POP_SUCCESS );
+}
+
+#endif  /* CYGWIN */
+
+
 /*----------------------------------------------- generic AUTH_USER */
 #    ifndef DECLARED_AUTH_USER 
 
@@ -1128,6 +1202,69 @@ auth_user ( POP *p, struct passwd *pw )
 #  include <rpc/key_prot.h>
 #endif /* SECURENISPLUS */
 
+/*
+ * do_ext_postauth_cmd: execute an external program on
+ * successful authentication. Arguments passed are IP address
+ * and user name. This is useful for SMTP after POP, and possibly
+ * other things.
+*/
+int
+do_ext_postauth_cmd ( POP *p )
+{
+
+#ifdef NEXT
+    int                    *status;
+#else
+    int                     status;
+#endif /* NEXT */
+
+    PID_T                   id, pid;
+    
+    DEBUG_LOG3 ( p, "...do_ext_postauth_cmd ext %s addr %s user %s",
+      p->ext_postauth_cmd, p->ipaddr, p->user);
+
+    /*  
+     * Send the message 
+     */
+    switch ( pid = fork() ) {
+        case 0:
+            /*  
+             * Open the log file 
+             */
+            (void) closelog();
+#ifdef SYSLOG42
+            (void) openlog ( p->myname, 0 );
+#else
+            (void) openlog ( p->myname, LOG_PID, p->log_facility );
+#endif
+            DEBUG_LOG3 ( p, "...do_ext_postauth_cmd ext %s addr %s user %s",
+                      p->ext_postauth_cmd, p->ipaddr, p->user);
+
+            (void) close  ( p->input_fd  );
+            (void) fclose ( p->output    );
+            (void) close  ( 0            );   /* close stdin so ext_postauth_cmd
+                                               * can open it.
+                                               */
+            (void) execl ( p->ext_postauth_cmd, p->ext_postauth_cmd,
+					p->ipaddr, p->user, NULLCP );
+            (void) _exit ( 1 );
+
+        case -1:
+            return pop_msg ( p, POP_FAILURE, HERE, "Unable to execute \"%s\": %s (%d)",
+                             p->ext_postauth_cmd, STRERROR(errno), errno );
+        default:
+
+            DEBUG_LOG1 ( p, "waiting on forked process (%u)",
+                         (unsigned) pid );
+#ifdef NEXT
+            while ( ( id = wait(status) ) >= 0 && id != pid );
+#else
+            id = waitpid ( pid, &status, 0 );
+#endif
+            return POP_SUCCESS;
+    }
+}
+
 pop_result 
 pop_pass ( POP *p )
 {
@@ -1331,6 +1468,9 @@ pop_pass ( POP *p )
     drac_it ( p );
 #endif /* DRAC_AUTH */
 
+    if ( p->ext_postauth_cmd != NULL )
+      do_ext_postauth_cmd( p );
+
     return ( pop_msg ( p, POP_SUCCESS, HERE,
                        "%s has %d visible message%s (%d hidden) in %ld octets.",
                         p->user,
