@@ -6,7 +6,7 @@
 
 
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
@@ -49,6 +49,9 @@
 
 #include <pwd.h>
 #include <netinet/in.h> /* For IPPORT_RESERVED */
+#if defined(INET6) && defined(HAVE_IRUSEROK_SA)
+#include <netdb.h>
+#endif
 
 #include "popper.h"
 #include "snprintf.h"
@@ -63,6 +66,9 @@ int pop_rpop (p)
      POP     *   p;
 {
     struct passwd  *   pw;
+#if defined(INET6) && defined(HAVE_IRUSEROK_SA)
+    struct addrinfo    hints, *res;
+#endif
 
     if ( p->ipport >= IPPORT_RESERVED || p->ipport < IPPORT_RESERVED/2 )  {
         pop_log ( p, POP_PRIORITY, HERE,
@@ -71,11 +77,33 @@ int pop_rpop (p)
         return ( pop_msg ( p, POP_FAILURE, HERE,
                            "Permission denied." ) );
     }
+#if defined(INET6) && defined(HAVE_IRUSEROK_SA)
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_flags = AI_PASSIVE | AI_NUMERICHOST;
+    hints.ai_family = PF_UNSPEC;
+    hints.ai_socktype = SOCK_STREAM;
+    if ( getaddrinfo ( p->ipaddr, NULL, &hints, &res ) != 0 )
+    {
+        DEBUG_LOG1 ( p, "cannot convert ip address user %s", p->user );
+        return ( pop_msg ( p, POP_FAILURE, HERE,
+                           "Permission denied." ) );
+    }
+    if ( iruserok_sa ( res->ai_addr, res->ai_addrlen,
+		       0, p->pop_parm[1], p->user ) != 0 )
+    {
+	freeaddrinfo ( res );
+        DEBUG_LOG1 ( p, "iruserok_sa denied user %s", p->user );
+        return ( pop_msg ( p, POP_FAILURE, HERE,
+                           "Permission denied." ) );
+    }
+    freeaddrinfo ( res );
+#else /* INET6 && HAVE_IRUSEROK_SA */
     if ( ruserok ( p->client, 0, p->pop_parm[1], p->user ) != 0 ) {
         DEBUG_LOG1 ( p, "ruserok denied user %s", p->user );
         return ( pop_msg ( p, POP_FAILURE, HERE,
                            "Permission denied." ) );
     }
+#endif /* INET6 && HAVE_IRUSEROK_SA */
     if ( p->nonauthfile != NULL && checknonauthfile ( p ) != 0 ) {
         return ( pop_msg ( p, POP_FAILURE, HERE,
                            "Permission denied." ) );
