@@ -6,7 +6,7 @@
  
  
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
@@ -606,15 +606,21 @@ pop_write ( POP *p, char *buffer, int length )
 {
 #ifdef    _DEBUG
     DEBUG_LOG4 ( p, "#chunky_writes=%d; tls_started=%d; %s %d bytes",
-                 (int) p->chunky_writes, p->tls_started,
+                 (int) p->chunky_writes, 
+                 p->tls_started,
                  ( p->chunky_writes == ChunkyAlways
                    || ( p->chunky_writes == ChunkyTLS
-                        && p->tls_started ) ) ? "buffering" : "sending",
+                        && p->tls_started )
+                   || ( p->chunky_writes == ChunkySASL
+                        && p->sasl_complete ) )
+                                               ? "buffering" : "sending",
                  length );
 #endif /* _DEBUG */
 
     if ( p->chunky_writes == ChunkyAlways
-         || ( p->chunky_writes == ChunkyTLS && p->tls_started ) )
+         || ( p->chunky_writes == ChunkyTLS  && p->tls_started   )
+         || ( p->chunky_writes == ChunkySASL && p->sasl_complete ) 
+       )
         pop_write_chunk ( p, buffer, length );
     else
         pop_write_now   ( p, buffer, length );
@@ -685,6 +691,8 @@ pop_write_flush ( POP *p )
 
     if ( p->tls_started ) {
         rslt = pop_tls_flush ( p->tls_context );
+    } else if ( p->sasl_complete ) {
+        rslt = pop_sasl_flush ( p );
     } else {
         rslt = fflush ( p->output );
     }
@@ -721,6 +729,8 @@ pop_write_now ( POP *p, char *buffer, int length )
 
     if ( p->tls_started ) {
         rslt = pop_tls_write ( p->tls_context, buffer, length );
+    } else if ( p->sasl_complete ) {
+        rslt = pop_sasl_write ( p, buffer, length );
     } else {
         rslt = fwrite ( buffer, 1, length,  p->output );
     }
@@ -733,7 +743,7 @@ pop_write_now ( POP *p, char *buffer, int length )
 #endif /* _DEBUG */
         }
     else { /* write failed */
-        if ( p->tls_started )
+        if ( p->tls_started || p->sasl_complete )
             pop_log ( p, POP_NOTICE, HERE, "Error writing to client" );
         else {
             int e = ferror ( p->output );
