@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
@@ -367,11 +367,10 @@ openssl_init ( pop_tls *pTLS, POP *pPOP )
     DEBUG_LOG1 ( pPOP, "...setting certificate file %s",
                  pPOP->tls_server_cert_file );
     nErr = SSL_CTX_use_certificate_chain_file ( pTLS->m_OpenSSLctx,
-                                          pPOP->tls_server_cert_file );
+                                                pPOP->tls_server_cert_file );
 
     if ( nErr <= 0 ) {
-        log_openssl_err ( pPOP, HERE, 
-                          "Error setting certificate PEM file %s",
+        log_openssl_err ( pPOP, HERE, "Error setting certificate PEM file %s",
                           pPOP->tls_server_cert_file );
         goto Done;
     }
