@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file license.txt specifies the terms for use, modification, or
  * redistribution.
  *
@@ -69,7 +69,7 @@ POP     *   p;
     int                     len    = 0;
 
     if ( p->parm_count == 1 ) {
-        len = strlen ( p->pop_parm[1] );
+        len = p->pop_parmlen[1];
 
         /*  
          * Convert the parameter into an integer 
@@ -184,7 +184,7 @@ POP     *   p;
     int                     len    = 0;
 
     if ( p->parm_count == 1 ) {
-        len = strlen ( p->pop_parm[1] );
+        len = p->pop_parmlen[1];
 
         /*  
          * Convert the parameter into an integer 
