@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
@@ -127,6 +127,22 @@
 
 extern char    *ERRMSG_PW;
 
+#ifdef     CYRUS_SASL
+static char    *ERRMSG_STRONG = "[AUTH] You must use stronger "
+                                 "authentication such as APOP or SASL"
+                                 "to connect to this server";
+static char    *ERRMSG_TLS_STRONG = "[AUTH] You must use TLS/SSL or "
+                                 "stronger authentication such as APOP "
+                                 "or SASL to connect to this server";
+#else  /* !CYRUS_SASL */
+static char    *ERRMSG_STRONG = "[AUTH] You must use stronger "
+                                 "authentication such as APOP"
+                                 "to connect to this server";
+static char    *ERRMSG_TLS_STRONG = "[AUTH] You must use TLS/SSL or "
+                                 "stronger authentication such as APOP "
+                                 "to connect to this server";
+#endif /*  CYRUS_SASL */
+
 
 /* 
  *  user:   Prompt for the user name at the start of a POP session
@@ -190,7 +206,7 @@ POP     * p;
     /*
      * Save the user name 
      */
-    user_name_len = strlen ( p->pop_parm[1] );
+    user_name_len = p->pop_parmlen[1];
     
     if ( user_name_len > 8 
          && !strcmp(p->pop_parm[1] + ( user_name_len - 8 ), "-no-mime" ) ) {
@@ -198,11 +214,11 @@ POP     * p;
         p->bNo_mime     = TRUE;
         user_name_len  -= 8;
         DEBUG_LOG5 ( p, "Turned on MIME mangling; "
-                        "user name entered was (%lu)'%.100s'; "
-                        "user name reset to (%lu)'%.*s'",
-                     strlen(p->pop_parm[1]),   p->pop_parm[1],
-                     (u_long) user_name_len,
-                     (int)user_name_len,       p->pop_parm[1] );
+                        "user name entered was (%d)'%.100s'; "
+                        "user name reset to (%zu)'%.*s'",
+                     p->pop_parmlen[1],   p->pop_parm[1],
+                     user_name_len,
+                     (int) user_name_len,       p->pop_parm[1] );
         *(p->pop_parm[1] + user_name_len) = '\0';
       }
     
@@ -229,13 +245,13 @@ POP     * p;
     if ( pw != NULL ) {
         p->pw        = *pw;                  /* copy it */
         p->pw.pw_dir = strdup(pw->pw_dir);   /* copy home directory */
-        DEBUG_LOG2 ( p, "home (%lu): '%s'",
+        DEBUG_LOG2 ( p, "home (%zu): '%s'",
                      strlen(p->pw.pw_dir), p->pw.pw_dir );
         /* pw_name is used by genpath so it must be copied or there are no
          * guarantees that it will be valid when genpath is called.
          */
          p->pw.pw_name = strdup(pw->pw_name); /* copy name */
-         DEBUG_LOG2 ( p, "name (%lu): '%s'",
+         DEBUG_LOG2 ( p, "name (%zu): '%s'",
                       strlen(p->pw.pw_name), p->pw.pw_name );
     }
     
@@ -370,14 +386,14 @@ POP     * p;
     
     if ( ( p->AllowClearText == ClearTextNever )    ||
          ( p->AllowClearText == ClearTextLocal &&
+#ifdef INET6
+           strcmp ( "::1", p->ipaddr ) != 0 &&
+#endif
            strncmp ( "127.", p->ipaddr, 4 ) != 0 )
        ) {
         DEBUG_LOG1 ( p, "AllowClearText=%u; (user not local)", 
                      p->AllowClearText );
-        return ( pop_auth_fail ( p, POP_FAILURE, HERE,
-                                 "[AUTH] You must use stronger "
-                                 "authentication such as APOP to "
-                                 "connect to this server" ) );
+        return ( pop_auth_fail ( p, POP_FAILURE, HERE, ERRMSG_STRONG ) );
     }
     if ( ( p->AllowClearText == ClearTextTLS &&
            p->tls_started == FALSE              )
@@ -385,9 +401,7 @@ POP     * p;
         DEBUG_LOG1 ( p, "AllowClearText=%u; (TLS/SSL not used)", 
                      p->AllowClearText );
         return ( pop_auth_fail ( p, POP_FAILURE, HERE,
-                                 "[AUTH] You must use TLS/SSL or "
-                                 "stronger authentication such as APOP "
-                                 "to connect to this server" ) );
+                                 ERRMSG_TLS_STRONG ) );
     }
 
 #if     defined(__bsdi__) && _BSDI_VERSION >= 199608
