@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (c) 2005 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
@@ -157,7 +157,7 @@ char *buffer;
     /*  
      * Look for initial period 
      */
-    DEBUG_LOG2 ( p, "Receiving (%lu): \"%.128s\"", strlen(buffer), buffer );
+    DEBUG_LOG2 ( p, "Receiving (%zu): \"%.128s\"", strlen(buffer), buffer );
     if ( buffer[0] == '.' ) {
         /*  
          * Exit on end of message 
@@ -228,8 +228,8 @@ POP *p;
                       p->user, p->client );
             (void) close  ( p->input_fd  );
             (void) fclose ( p->output    );
-            (void) close  ( 0            );   /* Make sure open return stdin.
-                                               * to be used by MAIL_COMMAND
+            (void) close  ( 0            );   /* close stin so MAIL_COMMAND
+                                               * can open it.
                                                */
             if ( open ( temp_xmit, O_RDONLY, 0 ) < 0 ) {
                 DEBUG_LOG3 ( p, "Unable to open \"%s\": %s (%d)",
