@@ -5,13 +5,16 @@
 
 
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
  *
  * Revisions:
  *
+ *    06/21/08  [Randall Gellens]
+ *           - Use new "uint4.h" for UINT4 typedef
+ *
  *    4/11/05  [Randall Gellens]
  *           - Open trace file as user, not root (fix security hole
  *             reported by Jens Steube)
@@ -21,6 +24,9 @@
  *    7/29/03  [Randall Gellens]
  *           - Removed prototype for srandom(); use stdlib.h instead.
  *
+ *    1/02/02  [Pierre Humblet]
+ *           - Added Cygwin specific code
+ *
  *   01/02/00  [Randall Gellens]
  *           - Added checks for dbm errors.
  *
@@ -88,6 +94,7 @@
  */
 
 #include "config.h"
+#include "uint4.h"
 
 #ifdef SCRAM
 #  include <md5.h>
@@ -135,6 +142,8 @@
 #  include <stdlib.h>
 #  define srandom srand
 #  define  random  rand
+#elif defined(__CYGWIN__)
+#  include <stdlib.h>
 #endif /* HAVE_SRANDOM */
 
 #include <flock.h>
@@ -889,8 +898,11 @@ char   *argv[];
         adios ( HERE,
                 "Only superuser or user \"%s\" can perform the requested function",
                 POPUID );
-
+#ifndef __CYGWIN__
     if ( myuid != 0 && ( initsw || dumpsw ) )
+#else
+    if ( popuser == FALSE && ( initsw || dumpsw ) )
+#endif
         adios ( HERE, "Only superuser can perform the requested function" );
 
     if ( delesw )
@@ -1021,16 +1033,16 @@ char   *argv[];
 #else
     db = dbm_open ( AUTHDB, O_RDWR | O_CREAT, 0600 );
     if ( db == NULL )
-        adios ( HERE, "unable to create POP authentication DB: %s (%d) [%d]", 
-                strerror(errno), errno, __LINE__ );
+        adios ( HERE, "unable to create POP authentication DB %s: %s (%d) [%d]", 
+                AUTHDB, strerror(errno), errno, __LINE__ );
     dbm_close ( db );
     if ( chown ( auth_dir, pop_pw.pw_uid, pop_pw.pw_gid ) == -1 
 #  ifndef BSD44_DBM
          || chown (auth_pag, pop_pw.pw_uid, pop_pw.pw_gid) == -1
 #  endif
        )
-        adios ( HERE, "error setting ownership of POP authentication DB: %s (%d) [%d]", 
-                strerror(errno), errno, __LINE__ );
+        adios ( HERE, "error setting ownership of POP authentication DB %s: %s (%d) [%d]", 
+                AUTHDB, strerror(errno), errno, __LINE__ );
 #endif /* GDBM */
 
     logit  ( trace_file, POP_NOTICE, HERE,
@@ -1097,7 +1109,7 @@ char   *argv[];
 
             if ( dumpsw ) {
                 printf ( "%.16s:%*s %.62s\n",
-                         key.dptr,
+                         (char *) key.dptr,
                          16 - (int)strlen(key.dptr), " ",
                          printable ( value.dptr, value.dsize ) );
                 BLATHER5 ( "Dumping \"%s\": (%d)'%s'; APOP %d; SCRAM %d",
@@ -1105,7 +1117,7 @@ char   *argv[];
                            value.dsize, printable ( value.dptr, value.dsize ),
                            apoplen, scramlen );
             } else {
-                printf ( "%-16s: %s %s\n", key.dptr,
+                printf ( "%-16s: %s %s\n", (char *) key.dptr,
                          ( apoplen  > 1 ) ? "APOP"  : "    ",
                          ( scramlen > 1 ) ? "SCRAM" : ""    );
                 BLATHER3 ( "Listing user \"%s\": APOP %s; SCRAM %s",
@@ -1136,7 +1148,7 @@ char   *argv[];
 
             if ( value.dptr == NULL ) {
                 printf   ( "%.16s:%*s *** no information?!?\n",
-                           key.dptr, 16 - (int)strlen(key.dptr), " " );
+                           (char *) key.dptr, 16 - (int)strlen(key.dptr), " " );
                 BLATHER1 ( "*** No information for user \"%s\"", buf );
             }
             else {
@@ -1150,7 +1162,7 @@ char   *argv[];
 
                 if ( dumpsw ) {
                     printf ( "%.16s:%*s %.62s\n",
-                             key.dptr,
+                             (char *) key.dptr,
                              16 - (int)strlen(key.dptr), " ",
                              printable ( value.dptr, value.dsize ) );
                     BLATHER5 ( "Dumping \"%s\": (%d)'%s'; APOP %d; SCRAM %d",
@@ -1158,7 +1170,7 @@ char   *argv[];
                                value.dsize, printable ( value.dptr, value.dsize ),
                                apoplen, scramlen );
                 } else {
-                    printf ( "%-16s: %s %s\n", key.dptr,
+                    printf ( "%-16s: %s %s\n", (char *) key.dptr,
                              ( apoplen  > 1 ) ? "APOP"  : "    ",
                              ( scramlen > 1 ) ? "SCRAM" : ""    );
                     BLATHER3 ( "Listing user \"%s\": APOP %s; SCRAM %s",
