@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (c) 2009 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2011 QUALCOMM Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  */
@@ -239,6 +239,14 @@ char        **  argv;
         DEBUG_LOG0 ( &p, "TLS Done" );   
     }
 
+/*
+ * If we have SASL support compiled in, initialize that now
+ */
+
+#ifdef    CYRUS_SASL
+    p.sasl.sasl__conn = pop_sasl_init ( &p );
+#endif /* CYRUS_SASL */
+
 /*  
  * Tell the user that we are listenting 
  */
@@ -389,6 +397,14 @@ DEBUG_LOG1 ( &p,"(v%s) Intro", VERSION );
     if ( p.tls_context != NULL )
         pop_tls_shutdown ( &p );
 
+    /*
+     * If we initialized any SASL stuff, clean it up
+     */
+#ifdef    CYRUS_SASL
+    if ( p.sasl.sasl__conn != NULL )
+        pop_sasl_done ( &p );
+#endif /* CYRUS_SASL */
+
     /*  
      * Log the end of activity 
      */
@@ -475,8 +491,9 @@ char
                               pPOP->pcInStart - pPOP->pcInBuf );
                 pPOP->pcInStart = pPOP->pcInEnd = pPOP->pcInBuf;
             }
+            pPOP->nLineLen = strlen(str);
             _DEBUG_LOG3 ( pPOP, "Qgetline() returning %d: '%.*s'",
-                          strlen(str), MIN(25, (int) strlen(str)), str );
+                          pPOP->nLineLen, MIN(25, pPOP->nLineLen), str );
             return ( str );
         } /* got a line */
 
@@ -506,6 +523,8 @@ char
             while ( TRUE ) {
                 if ( pPOP->tls_started )
                     len = pop_tls_read ( pPOP->tls_context, junk, sizeof(junk) );
+                else if ( pPOP->sasl_complete )
+                    len = pop_sasl_read ( pPOP, junk, sizeof(junk) );
                 else
                     len = read ( pPOP->input_fd, junk, sizeof(junk) );
                 if ( len <= 0 ) {
@@ -563,13 +582,16 @@ char
                 } /* found a '\n' */
             } /* loop and discard until we see a '\n' */
 
+            pPOP->nLineLen = strlen(str);
             _DEBUG_LOG2 ( pPOP, "Qgetline() returning %d ('%c')",
-                          strlen(str), *str );
+                          pPOP->nLineLen, *str );
             return ( str );
         } /* nRoom == 0 */
 
         if ( pPOP->tls_started )
             nRead = pop_tls_read ( pPOP->tls_context, pPOP->pcInEnd, nRoom );
+        else if ( pPOP->sasl_complete )
+            nRead = pop_sasl_read ( pPOP, pPOP->pcInEnd, nRoom );
         else
             nRead = read ( pPOP->input_fd,  pPOP->pcInEnd, nRoom );
 
@@ -586,6 +608,7 @@ char
             break;
     } /* main loop */
 
+    pPOP->nLineLen = 0;
     _DEBUG_LOG0 ( pPOP, "Qgetline() returning NULL" );
     return ( NULL );
 }
